@@ -272,11 +272,21 @@ int in_bad_region(struct detector *det, double fs, double ss)
 	ry = ys + p->cny;
 
 	for ( i=0; i<det->n_bad; i++ ) {
+
 		struct badregion *b = &det->bad[i];
-		if ( rx < b->min_x ) continue;
-		if ( rx > b->max_x ) continue;
-		if ( ry < b->min_y ) continue;
-		if ( ry > b->max_y ) continue;
+
+		if ( b->is_fsss ) {
+			if ( fs < b->min_fs ) continue;
+			if ( fs > b->max_fs ) continue;
+			if ( ss < b->min_ss ) continue;
+			if ( ss > b->max_ss ) continue;
+		} else {
+			if ( rx < b->min_x ) continue;
+			if ( rx > b->max_x ) continue;
+			if ( ry < b->min_y ) continue;
+			if ( ry > b->max_y ) continue;
+		}
+
 		return 1;
 	}
 
@@ -425,7 +435,7 @@ void record_image(struct image *image, int do_poisson, int background,
 }
 
 
-struct panel *find_panel(struct detector *det, double fs, double ss)
+signed int find_panel_number(struct detector *det, double fs, double ss)
 {
 	int p;
 
@@ -436,30 +446,21 @@ struct panel *find_panel(struct detector *det, double fs, double ss)
 		if ( (fs >= det->panels[p].min_fs)
 		  && (fs < det->panels[p].max_fs+1)
 		  && (ss >= det->panels[p].min_ss)
-		  && (ss < det->panels[p].max_ss+1) ) {
-			return &det->panels[p];
-		}
-	}
-
-	return NULL;
-}
-
-
-int find_panel_number(struct detector *det, int fs, int ss)
-{
-	int p;
-
-	for ( p=0; p<det->n_panels; p++ ) {
-		if ( (fs >= det->panels[p].min_fs)
-		  && (fs <= det->panels[p].max_fs)
-		  && (ss >= det->panels[p].min_ss)
-		  && (ss <= det->panels[p].max_ss) ) return p;
+		  && (ss < det->panels[p].max_ss+1) ) return p;
 	}
 
 	return -1;
 }
 
 
+struct panel *find_panel(struct detector *det, double fs, double ss)
+{
+	signed int pn = find_panel_number(det, fs, ss);
+	if ( pn == -1 ) return NULL;
+	return &det->panels[pn];
+}
+
+
 void fill_in_values(struct detector *det, struct hdfile *f)
 {
 	int i;
@@ -511,6 +512,11 @@ static struct badregion *new_bad_region(struct detector *det, const char *name)
 	new->max_x = NAN;
 	new->min_y = NAN;
 	new->max_y = NAN;
+	new->min_fs = 0;
+	new->max_fs = 0;
+	new->min_ss = 0;
+	new->max_ss = 0;
+	new->is_fsss = 0;
 	strcpy(new->name, name);
 
 	return new;
@@ -704,12 +710,36 @@ static int parse_field_bad(struct badregion *panel, const char *key,
 
 	if ( strcmp(key, "min_x") == 0 ) {
 		panel->min_x = atof(val);
+		if ( panel->is_fsss ) {
+			ERROR("You can't mix x/y and fs/ss in a bad region.\n");
+		}
 	} else if ( strcmp(key, "max_x") == 0 ) {
 		panel->max_x = atof(val);
+		if ( panel->is_fsss ) {
+			ERROR("You can't mix x/y and fs/ss in a bad region.\n");
+		}
 	} else if ( strcmp(key, "min_y") == 0 ) {
 		panel->min_y = atof(val);
+		if ( panel->is_fsss ) {
+			ERROR("You can't mix x/y and fs/ss in a bad region.\n");
+		}
 	} else if ( strcmp(key, "max_y") == 0 ) {
 		panel->max_y = atof(val);
+		if ( panel->is_fsss ) {
+			ERROR("You can't mix x/y and fs/ss in a bad region.\n");
+		}
+	} else if ( strcmp(key, "min_fs") == 0 ) {
+		panel->min_fs = atof(val);
+		panel->is_fsss = 1;
+	} else if ( strcmp(key, "max_fs") == 0 ) {
+		panel->max_fs = atof(val);
+		panel->is_fsss = 1;
+	} else if ( strcmp(key, "min_ss") == 0 ) {
+		panel->min_ss = atof(val);
+		panel->is_fsss = 1;
+	} else if ( strcmp(key, "max_ss") == 0 ) {
+		panel->max_ss = atof(val);
+		panel->is_fsss = 1;
 	} else {
 		ERROR("Unrecognised field '%s'\n", key);
 	}
@@ -1008,22 +1038,22 @@ struct detector *get_detector_geometry(const char *filename)
 
 	for ( i=0; i<det->n_bad; i++ ) {
 
-		if ( isnan(det->bad[i].min_x) ) {
+		if ( !det->bad[i].is_fsss && isnan(det->bad[i].min_x) ) {
 			ERROR("Please specify the minimum x coordinate for"
 			      " bad region %s\n", det->bad[i].name);
 			reject = 1;
 		}
-		if ( isnan(det->bad[i].min_y) ) {
+		if ( !det->bad[i].is_fsss && isnan(det->bad[i].min_y) ) {
 			ERROR("Please specify the minimum y coordinate for"
 			      " bad region %s\n", det->bad[i].name);
 			reject = 1;
 		}
-		if ( isnan(det->bad[i].max_x) ) {
+		if ( !det->bad[i].is_fsss && isnan(det->bad[i].max_x) ) {
 			ERROR("Please specify the maximum x coordinate for"
 			      " bad region %s\n", det->bad[i].name);
 			reject = 1;
 		}
-		if ( isnan(det->bad[i].max_y) ) {
+		if ( !det->bad[i].is_fsss && isnan(det->bad[i].max_y) ) {
 			ERROR("Please specify the maximum y coordinate for"
 			      " bad region %s\n", det->bad[i].name);
 			reject = 1;
@@ -1125,6 +1155,9 @@ struct detector *copy_geom(const struct detector *in)
 	out->bad = malloc(out->n_bad * sizeof(struct badregion));
 	memcpy(out->bad, in->bad, out->n_bad * sizeof(struct badregion));
 
+	out->n_rigid_groups = 0;
+	out->rigid_groups = NULL;
+
 	for ( i=0; i<out->n_panels; i++ ) {
 
 		struct panel *p;
@@ -1372,3 +1405,40 @@ int write_detector_geometry(const char *filename, struct detector *det)
 
 	return 0;
 }
+
+
+/**
+ * mark_resolution_range_as_bad:
+ * @image: An image structure
+ * @min: Minimum value of 1/d to be marked as bad
+ * @max: Maximum value of 1/d to be marked as bad
+ *
+ * Flags, in the bad pixel mask for @image, every pixel whose resolution is
+ * between @min and @max.
+ *
+ */
+
+void mark_resolution_range_as_bad(struct image *image,
+                                  double min, double max)
+{
+	int i;
+
+	for ( i=0; i<image->det->n_panels; i++ ) {
+
+		int fs, ss;
+		struct panel *p = &image->det->panels[i];
+
+		for ( ss=0; ss<p->h; ss++ ) {
+		for ( fs=0; fs<p->w; fs++ ) {
+			struct rvec q;
+			double r;
+			q = get_q_for_panel(p, fs, ss, NULL, 1.0/image->lambda);
+			r = modulus(q.u, q.v, q.w);
+			if ( (r >= min) && (r <= max) ) {
+				image->bad[i][fs+p->w*ss] = 1;
+			}
+		}
+		}
+
+	}
+}
