@@ -35,13 +35,20 @@
 
 #include "index.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 extern int run_dirax(struct image *image, IndexingPrivate *ipriv);
 
 extern IndexingPrivate *dirax_prepare(IndexingMethod *indm,
-                                      UnitCell *cell, const char *filename,
-                                      struct detector *det,
+                                      UnitCell *cell, struct detector *det,
                                       struct beam_params *beam, float *ltl);
 
 extern void dirax_cleanup(IndexingPrivate *pp);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif	/* DIRAX_H */
