@@ -61,6 +61,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <math.h>
+#include <pty.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/wait.h>
@@ -69,13 +70,12 @@
 #include <fcntl.h>
 #include <errno.h>
 
-#if HAVE_FORKPTY_LINUX
-#include <pty.h>
-#elif HAVE_FORKPTY_BSD
-#include <util.h>
+#ifdef HAVE_CLOCK_GETTIME
+#include <time.h>
+#else
+#include <sys/time.h>
 #endif
 
-
 #include "image.h"
 #include "mosflm.h"
 #include "utils.h"
@@ -831,8 +831,8 @@ int run_mosflm(struct image *image, IndexingPrivate *ipriv)
 
 
 IndexingPrivate *mosflm_prepare(IndexingMethod *indm, UnitCell *cell,
-                               const char *filename, struct detector *det,
-                               struct beam_params *beam, float *ltl)
+                                struct detector *det, struct beam_params *beam,
+                                float *ltl)
 {
 	struct mosflm_private *mp;
 	int need_cell = 0;
