@@ -11,6 +11,7 @@
  *   2010-2014 Thomas White <taw@physics.org>
  *   2011      Richard Kirian
  *   2011      Andrew Aquila
+ *   2014      Takanori Nakane <nakane.t@gmail.com>
  *
  * This file is part of CrystFEL.
  *
@@ -42,6 +43,7 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
+#include "version.h"
 #include "cell.h"
 #include "cell-utils.h"
 #include "utils.h"
@@ -316,6 +318,23 @@ static void write_stream_reflections_2_1(FILE *fh, RefList *list)
 }
 
 
+static int num_integrated_reflections(RefList *list)
+{
+	Reflection *refl;
+	RefListIterator *iter;
+	int n = 0;
+
+	for ( refl = first_refl(list, &iter);
+	      refl != NULL;
+	      refl = next_refl(refl, iter) )
+	{
+		if ( get_redundancy(refl) > 0 ) n++;
+	}
+
+	return n;
+}
+
+
 static void write_crystal(Stream *st, Crystal *cr, int include_reflections)
 {
 	UnitCell *cell;
@@ -324,6 +343,7 @@ static void write_crystal(Stream *st, Crystal *cr, int include_reflections)
 	double bsx, bsy, bsz;
 	double csx, csy, csz;
 	double a, b, c, al, be, ga;
+	double rad;
 
 	fprintf(st->fh, CRYSTAL_START_MARKER"\n");
 
@@ -332,17 +352,17 @@ static void write_crystal(Stream *st, Crystal *cr, int include_reflections)
 
 	cell_get_parameters(cell, &a, &b, &c, &al, &be, &ga);
 	fprintf(st->fh, "Cell parameters %7.5f %7.5f %7.5f nm,"
-		        " %7.5f %7.5f %7.5f deg\n",
-		        a*1.0e9, b*1.0e9, c*1.0e9,
-		        rad2deg(al), rad2deg(be), rad2deg(ga));
+	                " %7.5f %7.5f %7.5f deg\n",
+	                a*1.0e9, b*1.0e9, c*1.0e9,
+	                rad2deg(al), rad2deg(be), rad2deg(ga));
 
 	cell_get_reciprocal(cell, &asx, &asy, &asz,
 		                  &bsx, &bsy, &bsz,
 		                  &csx, &csy, &csz);
 	fprintf(st->fh, "astar = %+9.7f %+9.7f %+9.7f nm^-1\n",
-		asx/1e9, asy/1e9, asz/1e9);
+	        asx/1e9, asy/1e9, asz/1e9);
 	fprintf(st->fh, "bstar = %+9.7f %+9.7f %+9.7f nm^-1\n",
-		bsx/1e9, bsy/1e9, bsz/1e9);
+	        bsx/1e9, bsy/1e9, bsz/1e9);
 	fprintf(st->fh, "cstar = %+9.7f %+9.7f %+9.7f nm^-1\n",
 	        csx/1e9, csy/1e9, csz/1e9);
 
@@ -351,6 +371,9 @@ static void write_crystal(Stream *st, Crystal *cr, int include_reflections)
 	fprintf(st->fh, "centering = %c\n", cell_get_centering(cell));
 	fprintf(st->fh, "unique_axis = %c\n", cell_get_unique_axis(cell));
 
+	rad = crystal_get_profile_radius(cr);
+	fprintf(st->fh, "profile_radius = %.5f nm^-1\n", rad/1e9);
+
 	reflist = crystal_get_reflections(cr);
 	if ( reflist != NULL ) {
 
@@ -360,7 +383,7 @@ static void write_crystal(Stream *st, Crystal *cr, int include_reflections)
 		                1e10 / crystal_get_resolution_limit(cr));
 
 		fprintf(st->fh, "num_reflections = %i\n",
-		                num_reflections(reflist));
+		                num_integrated_reflections(reflist));
 		fprintf(st->fh, "num_saturated_reflections = %lli\n",
 		                crystal_get_num_saturated_reflections(cr));
 
@@ -408,11 +431,21 @@ void write_chunk(Stream *st, struct image *i, struct hdfile *hdfile,
 	fprintf(st->fh, "photon_energy_eV = %f\n",
 	        J_to_eV(ph_lambda_to_en(i->lambda)));
 
+	fprintf(st->fh, "beam_divergence = %.5f mrad\n", i->div*1e3);
+	fprintf(st->fh, "beam_bandwidth = %.5f %%\n", i->bw*100.0);
+
 	copy_hdf5_fields(hdfile, i->copyme, st->fh);
 
 	if ( i->det != NULL ) {
 
 		int j;
+		double tclen = 0.0;
+
+		for ( j=0; j<i->det->n_panels; j++ ) {
+			tclen += i->det->panels[j].clen;
+		}
+		fprintf(st->fh, "average_camera_length = %f m\n",
+		        tclen / i->det->n_panels);
 
 		for ( j=0; j<i->det->n_rigid_groups; j++ ) {
 
@@ -474,7 +507,7 @@ static int find_start_of_chunk(FILE *fh)
 }
 
 
-void read_crystal(Stream *st, struct image *image)
+static void read_crystal(Stream *st, struct image *image, StreamReadFlags srf)
 {
 	char line[1024];
 	char *rval = NULL;
@@ -500,7 +533,7 @@ void read_crystal(Stream *st, struct image *image)
 
 	do {
 
-		float u, v, w, lim;
+		float u, v, w, lim, rad;
 		char c;
 
 		rval = fgets(line, 1023, st->fh);
@@ -509,22 +542,30 @@ void read_crystal(Stream *st, struct image *image)
 		if ( rval == NULL ) break;
 
 		chomp(line);
-		if ( sscanf(line, "astar = %f %f %f", &u, &v, &w) == 3 ) {
+		if ( (srf & STREAM_READ_UNITCELL)
+		  && (sscanf(line, "astar = %f %f %f", &u, &v, &w) == 3) )
+		{
 			as.u = u*1e9;  as.v = v*1e9;  as.w = w*1e9;
 			have_as = 1;
 		}
 
-		if ( sscanf(line, "bstar = %f %f %f", &u, &v, &w) == 3 ) {
+		if ( (srf & STREAM_READ_UNITCELL)
+		  && (sscanf(line, "bstar = %f %f %f", &u, &v, &w) == 3) )
+		{
 			bs.u = u*1e9;  bs.v = v*1e9;  bs.w = w*1e9;
 			have_bs = 1;
 		}
 
-		if ( sscanf(line, "cstar = %f %f %f", &u, &v, &w) == 3 ) {
+		if ( (srf & STREAM_READ_UNITCELL)
+		  && (sscanf(line, "cstar = %f %f %f", &u, &v, &w) == 3) )
+		{
 			cs.u = u*1e9;  cs.v = v*1e9;  cs.w = w*1e9;
 			have_cs = 1;
 		}
 
-		if ( sscanf(line, "centering = %c", &c) == 1 ) {
+		if ( (srf & STREAM_READ_UNITCELL)
+		  && (sscanf(line, "centering = %c", &c) == 1) )
+		{
 			if ( !have_cen ) {
 				centering = c;
 				have_cen = 1;
@@ -533,7 +574,9 @@ void read_crystal(Stream *st, struct image *image)
 			}
 		}
 
-		if ( sscanf(line, "unique_axis = %c", &c) == 1 ) {
+		if ( (srf & STREAM_READ_UNITCELL)
+		  && (sscanf(line, "unique_axis = %c", &c) == 1) )
+		{
 			if ( !have_ua ) {
 				unique_axis = c;
 				have_ua = 1;
@@ -542,7 +585,9 @@ void read_crystal(Stream *st, struct image *image)
 			}
 		}
 
-		if ( strncmp(line, "lattice_type = ", 15) == 0 ) {
+		if ( (srf & STREAM_READ_UNITCELL)
+		  && (strncmp(line, "lattice_type = ", 15) == 0) )
+		{
 			if ( !have_latt ) {
 				lattice_type = lattice_from_str(line+15);
 				have_latt = 1;
@@ -561,7 +606,13 @@ void read_crystal(Stream *st, struct image *image)
 			crystal_set_resolution_limit(cr, lim*1e9);
 		}
 
-		if ( strcmp(line, REFLECTION_START_MARKER) == 0 ) {
+		if ( sscanf(line, "profile_radius = %f nm^-1", &rad) == 1 ) {
+			crystal_set_profile_radius(cr, rad*1e9);
+		}
+
+		if ( (strcmp(line, REFLECTION_START_MARKER) == 0)
+		  && (srf & STREAM_READ_REFLECTIONS) )
+		{
 
 			RefList *reflist;
 
@@ -612,6 +663,9 @@ void read_crystal(Stream *st, struct image *image)
 
 	}
 
+	/* Unused at the moment */
+	crystal_set_mosaicity(cr, 0.0);
+
 	/* Add crystal to the list for this image */
 	n = image->n_crystals+1;
 	crystals_new = realloc(image->crystals, n*sizeof(Crystal *));
@@ -627,7 +681,7 @@ void read_crystal(Stream *st, struct image *image)
 
 
 /* Read the next chunk from a stream and fill in 'image' */
-int read_chunk(Stream *st, struct image *image)
+int read_chunk_2(Stream *st, struct image *image,  StreamReadFlags srf)
 {
 	char line[1024];
 	char *rval = NULL;
@@ -641,7 +695,13 @@ int read_chunk(Stream *st, struct image *image)
 	image->crystals = NULL;
 	image->n_crystals = 0;
 
+	if ( (srf & STREAM_READ_REFLECTIONS) || (srf & STREAM_READ_UNITCELL) ) {
+		srf |= STREAM_READ_CRYSTALS;
+	}
+
 	do {
+		long long num_peaks;
+		float div, bw;
 
 		rval = fgets(line, 1023, st->fh);
 
@@ -668,6 +728,18 @@ int read_chunk(Stream *st, struct image *image)
 			have_ev = 1;
 		}
 
+		if ( sscanf(line, "beam_divergence = %f mrad", &div) == 1 ) {
+			image->div = div/1e3;
+		}
+
+		if ( sscanf(line, "beam_bandwidth = %f %%", &bw) == 1 ) {
+			image->bw = bw/100.0;
+		}
+
+		if ( sscanf(line, "num_peaks = %lld %%", &num_peaks) == 1 ) {
+			image->num_peaks = num_peaks;
+		}
+
 		if ( strncmp(line, "camera_length_", 14) == 0 ) {
 			if ( image->det != NULL ) {
 
@@ -694,15 +766,18 @@ int read_chunk(Stream *st, struct image *image)
 			}
 		}
 
-		if ( strcmp(line, PEAK_LIST_START_MARKER) == 0 ) {
+		if ( (srf & STREAM_READ_PEAKS)
+		  && (strcmp(line, PEAK_LIST_START_MARKER) == 0) ) {
 			if ( read_peaks(st->fh, image) ) {
 				ERROR("Failed while reading peaks\n");
 				return 1;
 			}
 		}
 
-		if ( strcmp(line, CRYSTAL_START_MARKER) == 0 ) {
-			read_crystal(st, image);
+		if ( (srf & STREAM_READ_CRYSTALS)
+		  && (strcmp(line, CRYSTAL_START_MARKER) == 0) )
+		{
+			read_crystal(st, image, srf);
 		}
 
 		/* A chunk must have at least a filename and a wavelength,
@@ -724,6 +799,14 @@ int read_chunk(Stream *st, struct image *image)
 }
 
 
+int read_chunk(Stream *st, struct image *image)
+{
+	return read_chunk_2(st, image, STREAM_READ_UNITCELL
+	                               | STREAM_READ_REFLECTIONS
+	                               | STREAM_READ_PEAKS);
+}
+
+
 void write_stream_header(FILE *ofh, int argc, char *argv[])
 {
 	int i;
@@ -779,6 +862,20 @@ Stream *open_stream_for_read(const char *filename)
 }
 
 
+/**
+ * open_stream_fd_for_write
+ * @fd: File descriptor (e.g. from open()) to use for stream data.
+ *
+ * Creates a new %Stream from @fd, so that stream data can be written to @fd
+ * using write_chunk().
+ *
+ * In contrast to open_stream_for_write(), this function does not write any of
+ * the usual headers.  This function is mostly for use when multiple substreams
+ * need to be multiplexed into a single master stream.  The master would be
+ * opened using open_stream_for_write(), and the substreams using this function.
+ *
+ * Returns: a %Stream, or NULL on failure.
+ */
 Stream *open_stream_fd_for_write(int fd)
 {
 	Stream *st;
@@ -795,25 +892,95 @@ Stream *open_stream_fd_for_write(int fd)
 	st->major_version = LATEST_MAJOR_VERSION;
 	st->minor_version = LATEST_MINOR_VERSION;
 
+	return st;
+}
+
+
+/**
+ * open_stream_for_write_2
+ * @filename: Filename of new stream
+ * @geom_filename: The geometry filename to copy
+ * @argc: The number of arguments to the program
+ * @argv: The arguments to the program
+ *
+ * Creates a new stream with name @filename, and adds the stream format
+ * and version header, plus a verbatim copy of the geometry file
+ *
+ * You may want to follow this with a call to write_command() to record the
+ * command line.
+ *
+ * Returns: a %Stream, or NULL on failure.
+ */
+Stream *open_stream_for_write_2(const char *filename,
+                                const char *geom_filename, int argc,
+                                char *argv[])
+
+{
+	Stream *st;
+
+	st = malloc(sizeof(struct _stream));
+	if ( st == NULL ) return NULL;
+
+	st->fh = fopen(filename, "w");
+	if ( st->fh == NULL ) {
+		ERROR("Failed to open stream.\n");
+		free(st);
+		return NULL;
+	}
+
+	st->major_version = LATEST_MAJOR_VERSION;
+	st->minor_version = LATEST_MINOR_VERSION;
+
 	fprintf(st->fh, "CrystFEL stream format %i.%i\n",
 	        st->major_version, st->minor_version);
+	fprintf(st->fh, "Generated by CrystFEL "CRYSTFEL_VERSIONSTRING"\n");
+	fflush(st->fh);
+
+	if ( (argc > 0) && (argv != NULL) ) {
+		write_command(st, argc, argv);
+	}
+	if ( geom_filename != NULL ) {
+		write_geometry_file(st, geom_filename);
+	}
 
 	return st;
 }
 
 
-
+/**
+ * open_stream_for_write
+ * @filename: Filename of new stream
+ *
+ * Creates a new stream with name @filename, and adds the stream format
+ * and version headers.
+ *
+ * You may want to follow this with a call to write_command() to record the
+ * command line.
+ *
+ * Returns: a %Stream, or NULL on failure.
+ */
 Stream *open_stream_for_write(const char *filename)
 {
-	int fd;
+	return open_stream_for_write_2(filename, NULL, 0, NULL);
+}
 
-	fd = open(filename, O_CREAT | O_TRUNC | O_WRONLY, S_IRUSR | S_IWUSR);
-	if ( fd == -1 ) {
-		ERROR("Failed to open stream.\n");
-		return NULL;
-	}
 
-	return open_stream_fd_for_write(fd);
+/**
+ * get_stream_fd
+ * @st: A %Stream
+ *
+ * This function gets the integer file descriptor for @st, a bit like fileno().
+ *
+ * This is useful in conjunction with open_stream_fd_for_write(), to get the
+ * underlying file descriptor to which the multiplexed stream data should be
+ * written.  In this case, the only other operations you should ever do (or have
+ * done) on @st are open_stream_for_write() and close_stream().
+ *
+ * Returns: an integer file descriptor
+ */
+int get_stream_fd(Stream *st)
+{
+	return fileno(st->fh);
 }
 
 
@@ -845,13 +1012,16 @@ int is_stream(const char *filename)
 }
 
 
-void write_line(Stream *st, const char *line)
-{
-	fprintf(st->fh, "%s\n", line);
-	fflush(st->fh);
-}
-
-
+/**
+ * write_command
+ * @st: A %Stream
+ * @argc: number of arguments
+ * @argv: command-line arguments
+ *
+ * Writes the command line to @st.  @argc and @argv should be exactly as were
+ * given to main().  This should usually be called immediately after
+ * open_stream_for_write().
+ */
 void write_command(Stream *st, int argc, char *argv[])
 {
 	int i;
@@ -863,6 +1033,40 @@ void write_command(Stream *st, int argc, char *argv[])
 		fprintf(st->fh, "%s", argv[i]);
 	}
 	fprintf(st->fh, "\n");
+	fflush(st->fh);
+}
+
+
+/**
+ * write_geometry_file
+ * @st: A %Stream
+ * @geom_filename: geomtry file name
+ *
+ * Writes the content of the geometry file to @st. This should usually be
+ * called immediately after write_command().
+ */
+void write_geometry_file(Stream *st, const char *geom_filename) {
+
+	char line[2014];
+	FILE *geom_fh;
+	char *rval;
+
+	if ( geom_filename == NULL ) return;
+
+	geom_fh = fopen(geom_filename, "r");
+	if ( geom_fh == NULL ) {
+		ERROR("Failed to read detector geometry from "
+		      "'%s'\n", geom_filename);
+		return;
+	}
+	fprintf(st->fh, GEOM_START_MARKER"\n");
+
+	do {
+		rval = fgets(line, 1023, geom_fh);
+		fputs(line, st->fh);
+	} while ( rval != NULL );
+	fprintf(st->fh, GEOM_END_MARKER"\n");
+	fflush(st->fh);
 }
 
 
