@@ -38,13 +38,20 @@
 #include "cell.h"
 #include "index.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 extern int run_xds(struct image *image, IndexingPrivate *ipriv);
 
 extern IndexingPrivate *xds_prepare(IndexingMethod *indm, UnitCell *cell,
-                                    const char *filename, struct detector *det,
+                                    struct detector *det,
                                     struct beam_params *beam, float *ltl);
 
 extern void xds_cleanup(IndexingPrivate *pp);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif	/* XDS_H */
