@@ -44,6 +44,7 @@
 #include <gsl/gsl_permutation.h>
 #include <gsl/gsl_randist.h>
 
+#include "version.h"
 #include <image.h>
 #include <utils.h>
 #include <symmetry.h>
@@ -63,6 +64,7 @@ static void show_help(const char *s)
 "\n"
 "  -h, --help                  Display this help message.\n"
 "\n"
+"      --version               Print CrystFEL version number and exit.\n"
 "  -o, --output=<filename>     Output stream.\n"
 "  -y, --symmetry=<sym>        Actual (\"target\") symmetry.\n"
 "  -w <sym>                    Apparent (\"source\" or \"twinned\") symmetry.\n"
@@ -76,6 +78,7 @@ static void show_help(const char *s)
 "      --ncorr=<n>             Use <n> correlations per crystal.  Default 1000\n"
 "  -j <n>                      Use <n> threads for CC calculation.\n"
 "      --really-random         Be non-deterministic.\n"
+"      --corr-matrix=<f>       Write the correlation matrix to file.\n"
 );
 }
 
@@ -659,6 +662,10 @@ static void write_reindexed_stream(const char *infile, const char *outfile,
 	FILE *fh;
 	FILE *ofh;
 	int i;
+	struct rvec as, bs, cs;
+	int have_as = 0;
+	int have_bs = 0;
+	int have_cs = 0;
 
 	fh = fopen(infile, "r");
 	if ( fh == NULL ) {
@@ -677,11 +684,96 @@ static void write_reindexed_stream(const char *infile, const char *outfile,
 
 		char *rval;
 		char line[1024];
+		int d = 0;
+		float u, v, w;
 
 		rval = fgets(line, 1023, fh);
 		if ( rval == NULL ) break;
 
-		fputs(line, ofh);
+		if ( strncmp(line, "Cell parameters ", 16) == 0 ) {
+			d = 1;
+		}
+
+		if ( sscanf(line, "astar = %f %f %f", &u, &v, &w) == 3 ) {
+			as.u = u*1e9;  as.v = v*1e9;  as.w = w*1e9;
+			have_as = 1;
+			d = 1;
+		}
+
+		if ( sscanf(line, "bstar = %f %f %f", &u, &v, &w) == 3 ) {
+			bs.u = u*1e9;  bs.v = v*1e9;  bs.w = w*1e9;
+			have_bs = 1;
+			d = 1;
+		}
+
+		if ( sscanf(line, "cstar = %f %f %f", &u, &v, &w) == 3 ) {
+			cs.u = u*1e9;  cs.v = v*1e9;  cs.w = w*1e9;
+			have_cs = 1;
+			d = 1;
+		}
+
+		if ( have_as && have_bs && have_cs ) {
+
+			UnitCell *cell;
+			double asx, asy, asz;
+			double bsx, bsy, bsz;
+			double csx, csy, csz;
+			double a, b, c, al, be, ga;
+
+			cell = cell_new_from_reciprocal_axes(as, bs, cs);
+
+			if ( assignments[i] ) {
+
+				signed int h, k, l;
+				struct rvec na, nb, nc;
+
+				get_equiv(amb, NULL, 0, 1, 0, 0, &h, &k, &l);
+				na.u = as.u*h + bs.u*k + cs.u*l;
+				na.v = as.v*h + bs.v*k + cs.v*l;
+				na.w = as.w*h + bs.w*k + cs.w*l;
+
+				get_equiv(amb, NULL, 0, 0, 1, 0, &h, &k, &l);
+				nb.u = as.u*h + bs.u*k + cs.u*l;
+				nb.v = as.v*h + bs.v*k + cs.v*l;
+				nb.w = as.w*h + bs.w*k + cs.w*l;
+
+				get_equiv(amb, NULL, 0, 0, 0, 1, &h, &k, &l);
+				nc.u = as.u*h + bs.u*k + cs.u*l;
+				nc.v = as.v*h + bs.v*k + cs.v*l;
+				nc.w = as.w*h + bs.w*k + cs.w*l;
+
+				cell_set_reciprocal(cell, na.u, na.v, na.w,
+				                          nb.u, nb.v, nb.w,
+				                          nc.u, nc.v, nc.w);
+
+			}
+
+			/* The cell parameters might change, so update them.
+			 * Unique axis, centering and lattice type can't change,
+			 * though. */
+			cell_get_parameters(cell, &a, &b, &c, &al, &be, &ga);
+			fprintf(ofh, "Cell parameters %7.5f %7.5f %7.5f nm,"
+			        " %7.5f %7.5f %7.5f deg\n",
+			        a*1.0e9, b*1.0e9, c*1.0e9,
+			        rad2deg(al), rad2deg(be), rad2deg(ga));
+
+			cell_get_reciprocal(cell, &asx, &asy, &asz,
+			                   &bsx, &bsy, &bsz,
+			                   &csx, &csy, &csz);
+			fprintf(ofh, "astar = %+9.7f %+9.7f %+9.7f nm^-1\n",
+			        asx/1e9, asy/1e9, asz/1e9);
+			fprintf(ofh, "bstar = %+9.7f %+9.7f %+9.7f nm^-1\n",
+			        bsx/1e9, bsy/1e9, bsz/1e9);
+			fprintf(ofh, "cstar = %+9.7f %+9.7f %+9.7f nm^-1\n",
+			        csx/1e9, csy/1e9, csz/1e9);
+
+			cell_free(cell);
+			have_as = 0;  have_bs = 0;  have_cs = 0;
+
+		}
+
+		/* Not a bug: REFLECTION_START_MARKER gets passed through */
+		if ( !d ) fputs(line, ofh);
 
 		if ( strcmp(line, REFLECTION_START_MARKER"\n") == 0 ) {
 			reindex_reflections(fh, ofh, assignments[i++], amb);
@@ -698,6 +790,110 @@ static void write_reindexed_stream(const char *infile, const char *outfile,
 }
 
 
+static void save_corr(const char *filename, struct cc_list *ccs, int n_crystals,
+                      int *assignments)
+{
+	hid_t fh, sh, dh;
+	hid_t ph;  /* Property list */
+	herr_t r;
+	hsize_t size[2];
+	float *matrix;
+	float *rmatrix;
+	int i;
+
+	matrix = malloc(n_crystals*n_crystals*sizeof(float));
+	rmatrix = malloc(n_crystals*n_crystals*sizeof(float));
+	if ( (matrix == NULL) || (rmatrix == NULL) ) return;
+
+	for ( i=0; i<n_crystals; i++ ) {
+
+		int k;
+
+		/* Set all values to zero */
+		for ( k=0; k<n_crystals; k++ ) {
+			matrix[i+n_crystals*k] = 0.0;
+			rmatrix[i+n_crystals*k] = 0.0;
+		}
+
+		/* CCs in current orientation */
+		k = 0;
+		do {
+			int j = ccs[i].ind[k];
+			if ( j == 0 ) break;
+			j -= 1;  /* Because we add one to use 0 as a marker */
+			matrix[i+n_crystals*j] = ccs[i].cc[k];
+			k++;
+		} while ( 1 );
+
+		/* CCs after reindexing */
+		k = 0;
+		do {
+			int j = ccs[i].ind_reidx[k];
+			if ( j == 0 ) break;
+			j -= 1;
+			rmatrix[i+n_crystals*j] = ccs[i].cc_reidx[k];
+			k++;
+		} while ( 1 );
+
+	}
+
+	fh = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
+	if ( fh < 0 ) {
+		ERROR("Couldn't create file: %s\n", filename);
+		return;
+	}
+
+	size[0] = n_crystals;
+	size[1] = n_crystals;
+	sh = H5Screate_simple(2, size, NULL);
+
+	/* Set compression */
+	ph = H5Pcreate(H5P_DATASET_CREATE);
+	H5Pset_chunk(ph, 2, size);
+	H5Pset_deflate(ph, 3);
+
+	dh = H5Dcreate2(fh, "correlation_matrix", H5T_NATIVE_FLOAT, sh,
+	                H5P_DEFAULT, ph, H5P_DEFAULT);
+	if ( dh < 0 ) {
+		ERROR("Couldn't create dataset\n");
+		H5Fclose(fh);
+		return;
+	}
+	r = H5Dwrite(dh, H5T_NATIVE_FLOAT, H5S_ALL,
+	             H5S_ALL, H5P_DEFAULT, matrix);
+	if ( r < 0 ) {
+		ERROR("Couldn't write data\n");
+		H5Dclose(dh);
+		H5Fclose(fh);
+		return;
+	}
+	H5Dclose(dh);
+
+	dh = H5Dcreate2(fh, "correlation_matrix_reindexed", H5T_NATIVE_FLOAT,
+	                sh, H5P_DEFAULT, ph, H5P_DEFAULT);
+	if ( dh < 0 ) {
+		ERROR("Couldn't create dataset\n");
+		H5Fclose(fh);
+		return;
+	}
+	r = H5Dwrite(dh, H5T_NATIVE_FLOAT, H5S_ALL,
+	             H5S_ALL, H5P_DEFAULT, rmatrix);
+	if ( r < 0 ) {
+		ERROR("Couldn't write data\n");
+		H5Dclose(dh);
+		H5Fclose(fh);
+		return;
+	}
+	H5Dclose(dh);
+
+	H5Fclose(fh);
+	free(matrix);
+	free(rmatrix);
+
+	STATUS("Wrote correlation matrix in HDF5 format to %s\n", filename);
+}
+
+
 int main(int argc, char *argv[])
 {
 	int c;
@@ -731,10 +927,12 @@ int main(int argc, char *argv[])
 	int n_threads = 1;
 	int config_random = 0;
 	char *operator = NULL;
+	char *corr_matrix_fn = NULL;
 
 	/* Long options */
 	const struct option longopts[] = {
 		{"help",               0, NULL,               'h'},
+		{"version",            0, NULL,               10 },
 		{"output",             1, NULL,               'o'},
 		{"symmetry",           1, NULL,               'y'},
 		{"iterations",         1, NULL,               'n'},
@@ -746,6 +944,7 @@ int main(int argc, char *argv[])
 		{"ncorr",              1, NULL,                6},
 		{"start-assignments",  1, NULL,                7},
 		{"operator",           1, NULL,                8},
+		{"corr-matrix",        1, NULL,                9},
 
 		{"really-random",      0, &config_random,      1},
 
@@ -763,6 +962,11 @@ int main(int argc, char *argv[])
 			show_help(argv[0]);
 			return 0;
 
+			case 10 :
+			printf("CrystFEL: " CRYSTFEL_VERSIONSTRING "\n");
+			printf(CRYSTFEL_BOILERPLATE"\n");
+			return 0;
+
 			case 'o' :
 			outfile = strdup(optarg);
 			break;
@@ -827,6 +1031,10 @@ int main(int argc, char *argv[])
 			operator = strdup(optarg);
 			break;
 
+			case 9 :
+			corr_matrix_fn = strdup(optarg);
+			break;
+
 			case 0 :
 			break;
 
@@ -1066,6 +1274,11 @@ int main(int argc, char *argv[])
 		detwin(ccs, n_crystals, assignments, fgfh, crystals);
 	}
 
+	if ( corr_matrix_fn != NULL ) {
+		save_corr(corr_matrix_fn, ccs, n_crystals, assignments);
+		free(corr_matrix_fn);
+	}
+
 	if ( fgfh != NULL ) {
 		fclose(fgfh);
 	}
