@@ -40,6 +40,7 @@
 #include <gdk/gdkkeysyms.h>
 #include <gsl/gsl_multifit_nlin.h>
 
+#include "version.h"
 #include "stream.h"
 #include "image.h"
 #include "utils.h"
@@ -55,6 +56,8 @@ static void show_help(const char *s)
 "Examine cell parameter histograms.\n"
 "\n"
 " -h, --help              Display this help message.\n"
+"     --version           Print CrystFEL version number and exit.\n"
+
 );
 }
 
@@ -883,8 +886,6 @@ static gint fit_sig(GtkWidget *widget, CellWindow *w)
 	if ( w->hist_be->show_sel ) fit_param(w->hist_be);
 	if ( w->hist_ga->show_sel ) fit_param(w->hist_ga);
 
-	/* FIXME: If all six worked, show values in cut+paste window */
-
 	redraw_all(w);
 
 	return TRUE;
@@ -1067,9 +1068,11 @@ static gint keypress_sig(GtkWidget *widget, GdkEventKey *event, HistoBox *h)
 
 	/* I'm too lazy to press shift */
 	if ( (event->keyval == GDK_plus) || (event->keyval == GDK_equal) ) {
-		h->n *= 2;
-		scan_cells(h->parent);
-		gtk_widget_queue_draw(h->da);
+		if ( h->n < 100000 ) {
+			h->n *= 2;
+			scan_cells(h->parent);
+			gtk_widget_queue_draw(h->da);
+		}
 	}
 
 	if ( (event->keyval == GDK_minus) && (h->n > 1) ) {
@@ -1214,6 +1217,7 @@ int main(int argc, char *argv[])
 	/* Long options */
 	const struct option longopts[] = {
 		{"help",               0, NULL,               'h'},
+		{"version",            0, NULL,                1 },
 		{0, 0, NULL, 0}
 	};
 
@@ -1227,6 +1231,11 @@ int main(int argc, char *argv[])
 			show_help(argv[0]);
 			return 0;
 
+			case 1 :
+			printf("CrystFEL: " CRYSTFEL_VERSIONSTRING "\n");
+			printf(CRYSTFEL_BOILERPLATE"\n");
+			return 0;
+
 			default :
 			return 1;
 
@@ -1264,29 +1273,22 @@ int main(int argc, char *argv[])
 
 		cur.det = NULL;
 
-		if ( read_chunk(st, &cur) != 0 ) {
+		if ( read_chunk_2(st, &cur, STREAM_READ_UNITCELL) != 0 ) {
 			break;
 		}
 
-		image_feature_list_free(cur.features);
-
 		for ( i=0; i<cur.n_crystals; i++ ) {
 
-			RefList *cr_refl;
 			Crystal *cr;
 
 			cr = cur.crystals[i];
 
-			cr_refl = crystal_get_reflections(cr);
-			reflist_free(cr_refl);
-
 			if ( w.n_cells == max_cells ) {
 
 				UnitCell **cells_new;
 				IndexingMethod *indms_new;
 				size_t nsz;
 
-
 				nsz = (max_cells+1024)*sizeof(UnitCell *);
 				cells_new = realloc(w.cells, nsz);
 				if ( cells_new == NULL ) {
@@ -1317,10 +1319,14 @@ int main(int argc, char *argv[])
 
 		}
 
-		fprintf(stderr, "Loaded %i cells from %i chunks\r", w.n_cells,
-		        ++n_chunks);
+		n_chunks++;
+		if ( n_chunks % 1000 == 0 ) {
+			fprintf(stderr, "Loaded %i cells from %i chunks\r",
+			        w.n_cells, n_chunks);
+		}
 
 	} while ( 1 );
+	fprintf(stderr, "Loaded %i cells from %i chunks\n", w.n_cells, n_chunks);
 
 	fprintf(stderr, "\n");
 
