@@ -3,13 +3,15 @@
  *
  * Quick yet non-crappy HDF viewer
  *
- * Copyright © 2012-2013 Deutsches Elektronen-Synchrotron DESY,
+ * Copyright © 2012-2014 Deutsches Elektronen-Synchrotron DESY,
  *                       a research centre of the Helmholtz Association.
  * Copyright © 2012 Richard Kirian
  *
  * Authors:
  *   2009-2013 Thomas White <taw@physics.org>
  *   2012      Richard Kirian
+ *   2014      Valerio Mariani
+ *   2014      Takanori Nakane
  *
  * This file is part of CrystFEL.
  *
@@ -39,6 +41,7 @@
 #include <string.h>
 #include <cairo.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gdk/gdkkeysyms.h>
 
 #include "dw-hdfsee.h"
 #include "hdfsee-render.h"
@@ -50,7 +53,6 @@
 #include "filters.h"
 
 
-/* FIXME: this trusts that 'message' doesn't contain any format strings */
 static void displaywindow_error(DisplayWindow *dw, const char *message)
 {
 	GtkWidget *window;
@@ -140,6 +142,116 @@ static void draw_panel_rectangle(cairo_t *cr, cairo_matrix_t *basic_m,
 }
 
 
+int render_adsc_uint16(DisplayWindow *dw, const char *filename)
+{
+	int x, y, fs, ss;
+	double dfs, dss;
+	int min_x = (int)dw->min_x;
+	int max_x = (int)dw->max_x;
+	int min_y = (int)dw->min_y;
+	int max_y = (int)dw->max_y;
+	int width = max_x - min_x;
+	int height = max_y - min_y;
+	unsigned short *buf;
+	struct image *image = dw->image;
+	FILE *fh;
+
+	if ( image == NULL ) return 1;
+	if ( image->det == NULL ) return 1;
+	if ( image->det->n_panels == 0)  return 1;
+
+	buf = calloc(width * height, sizeof(unsigned short));
+	if ( buf == NULL ) return 1;
+
+	fh = fopen(filename, "wb");
+	if ( fh == NULL ) {
+		free(buf);
+		return 1;
+	}
+
+	fprintf(fh, "{\n"
+		"HEADER_BYTES=512;\n"
+		"DIM=2;\n"
+		"BYTE_ORDER=little_endian;\n"
+		"TYPE=unsigned_short;\n"
+		"SIZE1=%d;\n"
+		"SIZE2=%d;\n"
+		"PIXEL_SIZE=%f;\n"
+		"WAVELENGTH=%f;\n"
+		"DISTANCE=%f;\n"
+		"PHI=0.0;\n"
+		"OSC_START=0.00;\n"
+		"OSC_END=0.00;\n"
+		"OSC_RANGE=0.00;\n"
+		"AXIS=phi;\n"
+		"BEAM_CENTER_X=%f;\n"
+		"BEAM_CENTER_Y=%f;\n"
+		"}\n",
+		width, height, 1 / image->det->panels[0].res * 10e2,
+		image->lambda * 10e9, image->det->panels[0].clen * 10e2,
+		-min_x / image->det->panels[0].res * 10e2,
+		-min_y / image->det->panels[0].res * 10e2);
+
+	fseek(fh, 512, SEEK_SET);
+
+	for ( y=min_y; y<max_y; y++ ) {
+	for ( x=min_x; x<max_x; x++ ) {
+
+		int val, invalid;
+		unsigned short out;
+
+		invalid = reverse_2d_mapping(x, y, &dfs, &dss, image->det);
+		if ( invalid ) continue;
+
+		fs = dfs;
+		ss = dss;
+		val = image->data[fs + image->width * ss];
+		if ( val < 0 ) {
+			out = 0;
+		} else if ( val > 65535 ) {
+			out = 65535;
+		} else {
+			out = val;
+		}
+
+		buf[(x - min_x) + (y - min_y) * width] = out;
+
+	}
+	}
+
+	fwrite(buf, sizeof(unsigned short), width * height, fh);
+	free(buf);
+	fclose(fh);
+
+	return 0;
+}
+
+
+static void draw_calib_focus_rectangle(cairo_t *cr, cairo_matrix_t *basic_m,
+                                       DisplayWindow *dw, int i)
+{
+	struct panel p = dw->image->det->panels[i];
+	int w = gdk_pixbuf_get_width(dw->pixbufs[i]);
+	int h = gdk_pixbuf_get_height(dw->pixbufs[i]);
+	cairo_matrix_t m;
+
+	/* Start with the basic coordinate system */
+	cairo_set_matrix(cr, basic_m);
+
+	/* Move to the right location */
+	cairo_translate(cr, p.cnx/dw->binning, p.cny/dw->binning);
+
+	/* Twiddle directions according to matrix */
+	cairo_matrix_init(&m, p.fsx, p.fsy, p.ssx, p.ssy, 0.0, 0.0);
+	cairo_transform(cr, &m);
+
+	cairo_set_line_width (cr, 3.0);
+	cairo_set_source_rgb (cr, 255, 255, 255);
+	cairo_rectangle(cr, 0.0, 0.0, w, h);
+
+}
+
+
 static void show_ring(cairo_t *cr, DisplayWindow *dw,
                       double d, const char *label, cairo_matrix_t *basic_m,
                       double r, double g, double b)
@@ -201,6 +313,38 @@ static void show_simple_ring(cairo_t *cr, DisplayWindow *dw,
 }
 
 
+static void maybe_draw_focus(DisplayWindow *dw, cairo_t *cr, int i,
+                             cairo_matrix_t *basic_m)
+{
+	switch ( dw->calib_mode ) {
+
+		case CALIBMODE_NONE:
+		break;
+
+		case CALIBMODE_PANELS:
+		if ( &dw->image->det->panels[i] == dw->calib_mode_curr_p) {
+			draw_calib_focus_rectangle(cr, basic_m, dw, i);
+			cairo_stroke(cr);
+		}
+		break;
+
+		case CALIBMODE_GROUPS:
+		if ( dw->image->det->panels[i].rigid_group == dw->calib_mode_curr_rg )
+		{
+			draw_calib_focus_rectangle(cr, basic_m, dw, i);
+			cairo_stroke(cr);
+		}
+		break;
+
+		case CALIBMODE_ALL:
+		draw_calib_focus_rectangle(cr, basic_m, dw, i);
+		cairo_stroke(cr);
+		break;
+
+	}
+}
+
+
 static int draw_stuff(cairo_surface_t *surf, DisplayWindow *dw)
 {
 	cairo_t *cr;
@@ -233,6 +377,10 @@ static int draw_stuff(cairo_surface_t *surf, DisplayWindow *dw)
 			draw_panel_rectangle(cr, &basic_m, dw, i);
 			cairo_fill(cr);
 
+			if ( dw->calib_mode && dw->calib_mode_show_focus ) {
+				maybe_draw_focus(dw, cr, i, &basic_m);
+			}
+
 		}
 
 	}
@@ -245,16 +393,6 @@ static int draw_stuff(cairo_surface_t *surf, DisplayWindow *dw)
 		cairo_set_source_rgb(cr, 1.0, 0.0, 0.0);
 		cairo_fill(cr);
 
-		//cairo_rectangle(cr, -370.0, 800.0, 120.0, 60.0);
-		//cairo_set_source_rgba(cr, 1.0, 0.0, 0.0, 0.3);
-		//cairo_fill(cr);
-
-		//show_simple_ring(cr, dw, 80.0, &basic_m);
-		//show_simple_ring(cr, dw, 186.0, &basic_m);
-		//show_simple_ring(cr, dw, 230.0, &basic_m);
-		//show_simple_ring(cr, dw, 320.0, &basic_m);
-		//show_simple_ring(cr, dw, 380.0, &basic_m);
-
 		/* Draw resolution circles */
 		if ( dw->n_rings == -1 ) {
 			/* n_rings == -1 means default behavior */
@@ -351,7 +489,7 @@ static int draw_stuff(cairo_surface_t *surf, DisplayWindow *dw)
 
 			cairo_set_line_width(cr, 0.75/dw->binning);
 			cairo_stroke(cr);
-
+;
 		}
 
 	}
@@ -380,7 +518,6 @@ static void redraw_window(DisplayWindow *dw)
 static void set_window_size(DisplayWindow *dw)
 {
 	gint width;
-	GdkGeometry geom;
 
 	if ( dw->image == NULL ) {
 		dw->width = 320;
@@ -405,7 +542,7 @@ static void set_window_size(DisplayWindow *dw)
 		dw->height = (max_y - min_y) / dw->binning;
 
 		/* Add a thin border */
-		dw->width += 2.0;
+		dw->width += 2.0;;
 		dw->height += 2.0;
 	}
 
@@ -414,13 +551,8 @@ static void set_window_size(DisplayWindow *dw)
 
 	gtk_widget_set_size_request(GTK_WIDGET(dw->drawingarea), width,
 				    dw->height);
-	geom.min_width = -1;
-	geom.min_height = -1;
-	geom.max_width = -1;
-	geom.max_height = -1;
-	gtk_window_set_geometry_hints(GTK_WINDOW(dw->window),
-				      GTK_WIDGET(dw->drawingarea), &geom,
-				      GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE);
+
+	gtk_window_resize(GTK_WINDOW(dw->window), width+30, dw->height+30);
 }
 
 
@@ -460,7 +592,7 @@ static void displaywindow_update(DisplayWindow *dw)
 
 
 static gboolean displaywindow_expose(GtkWidget *da, GdkEventExpose *event,
-				     DisplayWindow *dw)
+                                     DisplayWindow *dw)
 {
 	cairo_t *cr;
 
@@ -936,7 +1068,7 @@ static gint displaywindow_peaklist_response(GtkWidget *d, gint response,
 
 	gtk_widget_destroy(d);
 
-	return 0;
+	return 0;;
 }
 
 
@@ -1010,10 +1142,36 @@ static int load_geometry_file(DisplayWindow *dw, struct image *image,
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(w), TRUE);
 	dw->use_geom = 1;
 
+	w = gtk_ui_manager_get_widget(dw->ui,
+				      "/ui/displaywindow/tools/calibmode");
+	gtk_widget_set_sensitive(GTK_WIDGET(w), TRUE);
+
 	return 0;
 }
 
 
+static int save_geometry_file(DisplayWindow *dw)
+{
+	GtkWidget *d;
+	gchar * filename;
+	int w;
+
+	d = gtk_file_chooser_dialog_new("Save Detector Geometry",
+	                                GTK_WINDOW(dw->window),
+	                                GTK_FILE_CHOOSER_ACTION_SAVE,
+	                                GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                        GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+	                                NULL);
+
+	gtk_dialog_run (GTK_DIALOG (d));
+	filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER (d));
+	w = write_detector_geometry(filename, dw->image->det);
+	gtk_widget_destroy(d);
+	g_free(filename);
+	return w;
+}
+
+
 static gint displaywindow_loadgeom_response(GtkWidget *d, gint response,
                                             DisplayWindow *dw)
 {
@@ -1097,6 +1255,58 @@ static gint displaywindow_set_usegeom(GtkWidget *d, DisplayWindow *dw)
 	return 0;
 }
 
+static gint displaywindow_set_calibmode(GtkWidget *d, DisplayWindow *dw)
+{
+	GtkWidget *w, *vbox;
+	int val;
+
+	w =  gtk_ui_manager_get_widget(dw->ui,
+	                               "/ui/displaywindow/tools/calibmode");
+	if ( !dw->use_geom ) {
+		gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(w), 0);
+		return 0;
+	}
+
+	/* Get new value */
+	val = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(w));
+
+	/* When entering calibration mode */
+	if ( val ) {
+
+		guint cc;
+
+		if ( (dw->calib_mode_curr_rg == NULL)
+		  && (dw->calib_mode_curr_p == NULL) )
+		{
+			dw->calib_mode_curr_rg = dw->image->det->rigid_groups[0];
+			dw->calib_mode_curr_p = dw->calib_mode_curr_rg->panels[0];
+		}
+
+		dw->calib_mode = CALIBMODE_PANELS;
+
+		dw->statusbar = gtk_statusbar_new();
+		gtk_widget_show(dw->statusbar);
+		vbox = gtk_bin_get_child(GTK_BIN(dw->window));
+		gtk_box_pack_end(GTK_BOX(vbox), dw->statusbar,
+		                 FALSE, FALSE, 0);
+		cc = gtk_statusbar_get_context_id(GTK_STATUSBAR(dw->statusbar),
+		                                  "calibmode");
+		gtk_statusbar_push(GTK_STATUSBAR(dw->statusbar), cc,
+		                   "Calibration mode activated");
+		displaywindow_update(dw);
+
+	} else {
+
+		dw->calib_mode = CALIBMODE_NONE;
+		gtk_widget_destroy(dw->statusbar);
+		dw->statusbar = NULL;
+		displaywindow_update(dw);
+
+	}
+
+	return 0;
+}
+
 
 static gint displaywindow_set_rings(GtkWidget *d, DisplayWindow *dw)
 {
@@ -1140,6 +1350,8 @@ static gint displaywindow_save_response(GtkWidget *d, gint response,
 			r = render_tiff_fp(dw->image, file);
 		} else if ( type == 2 ) {
 			r = render_tiff_int16(dw->image, file, dw->boostint);
+		} else if (type == 3) {
+		  r = render_adsc_uint16(dw, file);
 		} else {
 			r = -1;
 		}
@@ -1202,6 +1414,9 @@ static gint displaywindow_save(GtkWidget *widget, DisplayWindow *dw)
 	gtk_combo_box_append_text(GTK_COMBO_BOX(cb),
 	       "TIFF - 16 bit signed integer "
 	       "(mono, unbinned, filtered, boosted)");
+	gtk_combo_box_append_text(GTK_COMBO_BOX(cb),
+	       "ADSC - 16 bit unsigned integer "
+	       "(unbinned, filtered, not boosted)");
 	gtk_combo_box_set_active(GTK_COMBO_BOX(cb), 0);
 
 	cd = malloc(sizeof(*cd));
@@ -1369,17 +1584,20 @@ static void numbers_update(DisplayWindow *dw)
 
 			val = dw->image->data[fs+ss*dw->image->width];
 
-			if ( val > 0.0 ) {
-				if ( log(val)/log(10.0) < 5 ) {
-					snprintf(s, 31, "%.0f", val);
-				} else {
-					snprintf(s, 31, "HUGE");
-				}
+			if ( (val > 0.0) && (log(val)/log(10.0) >= 5) ) {
+				snprintf(s, 31, "HUGE");
+			} else if ( (val < 0.0) && (log(-val)/log(10) >= 4) ) {
+				snprintf(s, 31, "-HUGE");
 			} else {
-				if ( log(-val)/log(10) < 4 ) {
-					snprintf(s, 31, "%.0f", val);
-				} else {
-					snprintf(s, 31, "-HUGE");
+				size_t l, i;
+				snprintf(s, 31, "%.4f", val);
+				l = strlen(s);
+				for ( i=l-1; i>0; i-- ) {
+					if ( s[i] == '0' ) s[i] = '\0';
+					if ( s[i] == '.' ) {
+						s[i] = '\0';
+						break;
+					}
 				}
 			}
 
@@ -1403,7 +1621,7 @@ static void numbers_update(DisplayWindow *dw)
 		char text[64];
 
 		f = image_feature_closest(dw->image->features, ffs, fss,
-		                          &dmin, &imin);
+		                          &dmin, &imin, dw->image->det);
 		if ( dmin < dw->ring_radius*dw->binning ) {
 			strncpy(text, f->name, 32);
 		} else {
@@ -1496,6 +1714,8 @@ static void displaywindow_addmenubar(DisplayWindow *dw, GtkWidget *vbox,
 	GtkToggleActionEntry toggles[] = {
 		{ "GeometryAction", NULL, "Use Detector Geometry", NULL, NULL,
 			G_CALLBACK(displaywindow_set_usegeom), FALSE },
+		{ "CalibModeAction", NULL, "Calibration Mode", NULL, NULL,
+			G_CALLBACK(displaywindow_set_calibmode), FALSE },
 		{ "ColScaleAction", NULL, "Colour Scale", NULL, NULL,
 			G_CALLBACK(displaywindow_set_colscale), FALSE },
 		{ "RingsAction", NULL, "Resolution Rings", "F9", NULL,
@@ -1605,10 +1825,15 @@ static gint displaywindow_newhdf(GtkMenuItem *item, struct newhdf *nh)
 		w = gtk_ui_manager_get_widget(nh->dw->ui,
 				      "/ui/displaywindow/view/usegeom");
 		gtk_widget_set_sensitive(GTK_WIDGET(w), FALSE);
+
 		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(w), FALSE);
 		nh->dw->use_geom = 0;
 		nh->dw->image->det = nh->dw->simple_geom;
 
+		w = gtk_ui_manager_get_widget(nh->dw->ui,
+					   "/ui/displaywindow/tools/calibmode");
+		gtk_widget_set_sensitive(GTK_WIDGET(w), FALSE);
+
 	}
 
 	if ( nh->dw->use_geom ) {
@@ -1791,6 +2016,38 @@ static gint displaywindow_motion(GtkWidget *widget, GdkEventMotion *event,
 }
 
 
+static void calibmode_press(DisplayWindow *dw, GdkEventButton *event)
+{
+	int x,y;
+	double dfs, dss;
+	int fs, ss, revmap_return_value;
+	char statusbar_string[80];
+	guint cc;
+
+	cc = gtk_statusbar_get_context_id(GTK_STATUSBAR(dw->statusbar),
+	                                  "calibmode");
+
+	x = dw->binning * (event->x);
+	y = dw->binning * (dw->height - 1 - event->y);
+	x += dw->min_x;
+	y += dw->min_x;
+	snprintf(statusbar_string, 80,
+	         "Last clicked position: x: %i, y: %i (Not in panel)", x, y);
+
+	revmap_return_value = reverse_2d_mapping(x, y, &dfs, &dss,
+	                                         dw->image->det);
+	if ( revmap_return_value == 0 ) {
+		fs = dfs;
+		ss = dss;
+		snprintf(statusbar_string, 80,
+                        "Last clicked position: x: %i, y: %i, fs: %u, ss: %u,"
+                        " (panel %s)",
+                        x, y, fs, ss, find_panel(dw->image->det, fs, ss)->name);
+	}
+	gtk_statusbar_push(GTK_STATUSBAR(dw->statusbar), cc, statusbar_string);
+}
+
+
 static gint displaywindow_press(GtkWidget *widget, GdkEventButton *event,
                                 DisplayWindow *dw)
 {
@@ -1812,16 +2069,369 @@ static gint displaywindow_press(GtkWidget *widget, GdkEventButton *event,
 			numbers_update(dw);
 		}
 
+		if ( dw->statusbar != NULL ) {
+			calibmode_press(dw, event);
+		}
+
 	}
 
 	return 0;
+}
 
+
+static int curr_rg_pointer_index(DisplayWindow *dw)
+{
+	int r;
+
+	for ( r=0; r<dw->image->det->n_rigid_groups; ++r) {
+		if ( dw->image->det->rigid_groups[r] == dw->calib_mode_curr_rg ) {
+			return r;
+		}
+	}
+
+	/* Never reached (we hope) */
+	return 999;
+}
+
+
+static int curr_p_pointer_index(DisplayWindow *dw)
+{
+	int p;
+
+	for ( p=0; p<dw->image->det->n_panels; ++p) {
+		if ( &dw->image->det->panels[p] == dw->calib_mode_curr_p ) {
+			return p;
+		}
+	}
+
+	/* Never reached (we hope) */
+	return 999;
+}
+
+
+static void select_next_group(DisplayWindow *dw, int num_rg)
+{
+	if ( dw->calib_mode_curr_rg == dw->image->det->rigid_groups[num_rg-1] ) {
+		dw->calib_mode_curr_rg = dw->image->det->rigid_groups[0];
+	} else {
+		dw->calib_mode_curr_rg =
+		  dw->image->det->rigid_groups[curr_rg_pointer_index(dw)+1];
+	}
+}
+
+
+static void select_prev_group(DisplayWindow *dw, int num_rg)
+{
+	if ( dw->calib_mode_curr_rg == dw->image->det->rigid_groups[0] ) {
+		dw->calib_mode_curr_rg = dw->image->det->rigid_groups[num_rg-1];
+	} else {
+		dw->calib_mode_curr_rg =
+		  dw->image->det->rigid_groups[curr_rg_pointer_index(dw)-1];
+	}
+}
+
+
+static void select_next_panel(DisplayWindow *dw, int num_p)
+{
+	if ( dw->calib_mode_curr_p == &dw->image->det->panels[num_p-1] ) {
+		dw->calib_mode_curr_p = &dw->image->det->panels[0];
+	} else {
+		dw->calib_mode_curr_p =
+		  &dw->image->det->panels[curr_p_pointer_index(dw)+1];
+	}
+}
+
+
+static void select_prev_panel(DisplayWindow *dw, int num_p)
+{
+	if ( dw->calib_mode_curr_p == &dw->image->det->panels[0] ) {
+		dw->calib_mode_curr_p = &dw->image->det->panels[num_p-1];
+	} else {
+		dw->calib_mode_curr_p =
+		  &dw->image->det->panels[curr_p_pointer_index(dw)-1];
+	}
+}
+
+
+static void toggle_calibmode_groupmode(DisplayWindow *dw)
+{
+	struct rigid_group *rg;
+	struct detector *det = dw->image->det;
+
+	switch ( dw->calib_mode ) {
+
+		case CALIBMODE_NONE:
+		break;
+
+		case CALIBMODE_PANELS:
+		if ( det->n_rigid_groups != det->n_panels ) {
+			/* Only change if there are any rigid groups defined */
+			dw->calib_mode = CALIBMODE_GROUPS;
+			rg = dw->calib_mode_curr_p->rigid_group;
+			dw->calib_mode_curr_rg = rg;
+		} else {
+			/* ...otherwise skip to ALL mode */
+			dw->calib_mode = CALIBMODE_ALL;
+		}
+		break;
+
+		case CALIBMODE_GROUPS:
+		dw->calib_mode = CALIBMODE_ALL;
+		break;
+
+		case CALIBMODE_ALL:
+		dw->calib_mode = CALIBMODE_PANELS;
+		dw->calib_mode_curr_p = dw->calib_mode_curr_rg->panels[0];
+		break;
+
+	}
+}
+
+
+static void calibmode_next(DisplayWindow *dw)
+{
+	int n;
+
+	switch ( dw->calib_mode ) {
+
+		case CALIBMODE_NONE:
+		break;
+
+		case CALIBMODE_PANELS:
+		n = dw->image->det->n_panels;
+		select_next_panel(dw, n);
+		break;
+
+		case CALIBMODE_GROUPS:
+		n = dw->image->det->n_rigid_groups;
+		select_next_group(dw, n);
+		break;
+
+		case CALIBMODE_ALL:
+		break;
+
+	}
+}
+
+
+static void calibmode_prev(DisplayWindow *dw)
+{
+	int n;
+
+	switch ( dw->calib_mode ) {
+
+		case CALIBMODE_NONE:
+		break;
+
+		case CALIBMODE_PANELS:
+		n = dw->image->det->n_panels;
+		select_prev_panel(dw, n);
+		break;
+
+		case CALIBMODE_GROUPS:
+		n = dw->image->det->n_rigid_groups;
+		select_prev_group(dw, n);
+		break;
+
+		case CALIBMODE_ALL:
+		break;
+
+	}
+}
+
+
+static void calibmode_up(DisplayWindow *dw)
+{
+	int pi;
+
+	switch ( dw->calib_mode ) {
+
+		case CALIBMODE_NONE:
+		break;
+
+		case CALIBMODE_PANELS:
+		dw->calib_mode_curr_p->cny += 1.0;
+		break;
+
+		case CALIBMODE_GROUPS:
+		for ( pi=0; pi<dw->calib_mode_curr_rg->n_panels; ++pi ) {
+			dw->calib_mode_curr_rg->panels[pi]->cny += 1.0;
+		}
+		break;
+
+		case CALIBMODE_ALL:
+		for ( pi=0; pi<dw->image->det->n_panels; ++pi ) {
+			dw->image->det->panels[pi].cny += 1.0;
+		}
+		break;
+
+	}
+}
+
+
+static void calibmode_down(DisplayWindow *dw)
+{
+	int pi;
+
+	switch ( dw->calib_mode ) {
+
+		case CALIBMODE_NONE:
+		break;
+
+		case CALIBMODE_PANELS:
+		dw->calib_mode_curr_p->cny -= 1.0;
+		break;
+
+		case CALIBMODE_GROUPS:
+		for ( pi=0; pi<dw->calib_mode_curr_rg->n_panels; ++pi ) {
+			dw->calib_mode_curr_rg->panels[pi]->cny -= 1.0;
+		}
+		break;
+
+		case CALIBMODE_ALL:
+		for ( pi=0; pi<dw->image->det->n_panels; ++pi ) {
+			dw->image->det->panels[pi].cny -= 1.0;
+		}
+		break;
+
+	}
+}
+
+
+static void calibmode_left(DisplayWindow *dw)
+{
+	int pi;
+
+	switch ( dw->calib_mode ) {
+
+		case CALIBMODE_NONE:
+		break;
+
+		case CALIBMODE_PANELS:
+		dw->calib_mode_curr_p->cnx -= 1.0;
+		break;
+
+		case CALIBMODE_GROUPS:
+		for ( pi=0; pi<dw->calib_mode_curr_rg->n_panels; ++pi ) {
+			dw->calib_mode_curr_rg->panels[pi]->cnx -= 1.0;
+		}
+		break;
+
+		case CALIBMODE_ALL:
+		for ( pi=0; pi<dw->image->det->n_panels; ++pi ) {
+			dw->image->det->panels[pi].cnx -= 1.0;
+		}
+		break;
+
+	}
+}
+
+
+static void calibmode_right(DisplayWindow *dw)
+{
+	int pi;
+
+	switch ( dw->calib_mode ) {
+
+		case CALIBMODE_NONE:
+		break;
+
+		case CALIBMODE_PANELS:
+		dw->calib_mode_curr_p->cnx += 1.0;
+		break;
+
+		case CALIBMODE_GROUPS:
+		for ( pi=0; pi<dw->calib_mode_curr_rg->n_panels; ++pi ) {
+			dw->calib_mode_curr_rg->panels[pi]->cnx += 1.0;
+		}
+		break;
+
+		case CALIBMODE_ALL:
+		for ( pi=0; pi<dw->image->det->n_panels; ++pi ) {
+			dw->image->det->panels[pi].cnx += 1.0;
+		}
+		break;
+
+	}
+}
+
+
+static gint displaywindow_keypress(GtkWidget *widget, GdkEventKey *event,
+                                   DisplayWindow *dw)
+{
+	int s;
+
+	if ( !dw->calib_mode ) {
+		return 0;
+	}
+
+	switch ( event->keyval ) {
+
+		case GDK_Up:
+		case GDK_KP_Up:
+		calibmode_up(dw);
+		redraw_window(dw);
+		break;
+
+		case GDK_Down:
+		case GDK_KP_Down:
+		calibmode_down(dw);
+		redraw_window(dw);
+		break;
+
+		case GDK_Left:
+		case GDK_KP_Left:
+		calibmode_left(dw);
+		redraw_window(dw);
+		break;
+
+		case GDK_Right:
+		case GDK_KP_Right:
+		calibmode_right(dw);
+		redraw_window(dw);
+		break;
+
+		case GDK_plus:
+		case GDK_KP_Add:
+		calibmode_next(dw);
+		redraw_window(dw);
+		break;
+
+		case GDK_minus:
+		case GDK_KP_Subtract:
+		calibmode_prev(dw);
+		redraw_window(dw);
+		break;
+
+		case GDK_f:
+		dw->calib_mode_show_focus = 1 - dw->calib_mode_show_focus;
+		redraw_window(dw);
+		break;
+
+		case GDK_g:
+		toggle_calibmode_groupmode(dw);
+		redraw_window(dw);
+		break;
+
+		case GDK_s:
+		s = save_geometry_file(dw);
+		if ( s != 0 ) {
+			if ( s != 2 ) {
+				displaywindow_error(dw,
+				       "Unable to save the detector geometry.");
+			}
+		}
+		break;
+
+	}
+
+	return 0;
 }
 
 
 DisplayWindow *displaywindow_open(const char *filename, const char *peaks,
                                   double boost, int binning,
-                                  int noisefilter, int colscale,
+                                  int noisefilter, int calibmode, int colscale,
                                   const char *element, const char *geometry,
                                   const char *beam,
                                   int show_rings, double *ring_radii,
@@ -1832,6 +2442,7 @@ DisplayWindow *displaywindow_open(const char *filename, const char *peaks,
 	char *title;
 	GtkWidget *vbox;
 	GtkWidget *w;
+	GtkWidget *ww;
 
 	dw = calloc(1, sizeof(DisplayWindow));
 	if ( dw == NULL ) return NULL;
@@ -1858,6 +2469,11 @@ DisplayWindow *displaywindow_open(const char *filename, const char *peaks,
 	dw->n_rings = n_rings;
 	dw->median_filter = median_filter;
 	dw->image = calloc(1, sizeof(struct image));
+	dw->calib_mode = CALIBMODE_NONE;
+	dw->calib_mode_curr_rg = NULL;
+	dw->calib_mode_curr_p = NULL;
+	dw->calib_mode_show_focus = 1;
+	dw->statusbar = NULL;
 
 	if ( beam != NULL ) {
 		dw->image->beam = get_beam_parameters(beam);
@@ -1897,7 +2513,6 @@ DisplayWindow *displaywindow_open(const char *filename, const char *peaks,
 		free(dw);
 		return NULL;
 	}
-
 	dw->loaded_geom = NULL;
 	dw->simple_geom = simple_geometry(dw->image);
 	dw->image->det = dw->simple_geom;
@@ -1927,12 +2542,18 @@ DisplayWindow *displaywindow_open(const char *filename, const char *peaks,
 	displaywindow_addmenubar(dw, vbox, colscale);
 
 	dw->drawingarea = gtk_drawing_area_new();
-	gtk_box_pack_start(GTK_BOX(vbox), dw->drawingarea, TRUE, TRUE, 0);
+	dw->scrollarea = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(dw->scrollarea),
+	                               GTK_POLICY_AUTOMATIC,
+	                               GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(dw->scrollarea),
+	                                      dw->drawingarea);
+	gtk_box_pack_start(GTK_BOX(vbox), dw->scrollarea, TRUE, TRUE, 0);
 
 	g_signal_connect(GTK_OBJECT(dw->drawingarea), "expose-event",
 			 G_CALLBACK(displaywindow_expose), dw);
 
-	gtk_window_set_resizable(GTK_WINDOW(dw->window), FALSE);
+	gtk_window_set_resizable(GTK_WINDOW(dw->window), TRUE);
 	gtk_widget_show_all(dw->window);
 
 	w = gtk_ui_manager_get_widget(dw->ui, "/ui/displaywindow/view/usegeom");
@@ -1941,18 +2562,37 @@ DisplayWindow *displaywindow_open(const char *filename, const char *peaks,
 		load_geometry_file(dw, dw->image, geometry);
 	}
 
+	if ( dw->use_geom ) {
+		dw->calib_mode = calibmode;
+	} else {
+		w = gtk_ui_manager_get_widget(dw->ui,
+		                           "/ui/displaywindow/tools/calibmode");
+		gtk_widget_set_sensitive(GTK_WIDGET(w), FALSE);
+	}
+
+	if ( dw->calib_mode ) {
+		ww = gtk_ui_manager_get_widget(dw->ui,
+		                           "/ui/displaywindow/tools/calibmode");
+		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(ww), TRUE);
+	}
+
 	displaywindow_update(dw);
 
 	gtk_widget_add_events(GTK_WIDGET(dw->drawingarea),
 	                      GDK_BUTTON_PRESS_MASK
 	                      | GDK_BUTTON_RELEASE_MASK
-	                      | GDK_BUTTON1_MOTION_MASK);
+	                      | GDK_BUTTON1_MOTION_MASK
+	                      | GDK_KEY_PRESS_MASK);
 	g_object_set(G_OBJECT(dw->drawingarea), "can-focus", TRUE, NULL);
 
+	gtk_widget_grab_focus(dw->drawingarea);
+
 	g_signal_connect(GTK_OBJECT(dw->drawingarea), "button-press-event",
 	                 G_CALLBACK(displaywindow_press), dw);
 	g_signal_connect(GTK_OBJECT(dw->drawingarea), "button-release-event",
 	                 G_CALLBACK(displaywindow_release), dw);
+	g_signal_connect(GTK_OBJECT(dw->drawingarea), "key-press-event",
+	                 G_CALLBACK(displaywindow_keypress), dw);
 
 	displaywindow_update_menus(dw, element);
 	dw->not_ready_yet = 0;
