@@ -3,11 +3,11 @@
  *
  * Small program to manipulate reflection lists
  *
- * Copyright © 2013 Deutsches Elektronen-Synchrotron DESY,
- *                  a research centre of the Helmholtz Association.
+ * Copyright © 2013-2014 Deutsches Elektronen-Synchrotron DESY,
+ *                       a research centre of the Helmholtz Association.
  *
  * Authors:
- *   2009-2013 Thomas White <taw@physics.org>
+ *   2009-2014 Thomas White <taw@physics.org>
  *
  * This file is part of CrystFEL.
  *
@@ -38,6 +38,7 @@
 #include <unistd.h>
 #include <getopt.h>
 
+#include "version.h"
 #include "utils.h"
 #include "reflist-utils.h"
 #include "symmetry.h"
@@ -53,6 +54,7 @@ static void show_help(const char *s)
 "Manipulate reflection lists.\n"
 "\n"
 "  -h, --help                 Display this help message.\n"
+"      --version              Print CrystFEL version number and exit.\n"
 "\n"
 "  -i, --input=<file>         Read reflections from <file>.\n"
 "  -y, --symmetry=<sym>       The symmetry of the input reflection list.\n"
@@ -73,6 +75,9 @@ static void show_help(const char *s)
 "      --no-need-all-parts    Output a twinned reflection even if not all\n"
 "                              the necessary equivalents were present.\n"
 "\n"
+"You can reindex the reflections according to an operation, e.g. k,h,-l:\n"
+"      --reindex=<op>         Reindex according to <op>.\n"
+"\n"
 "Use this option with care, and only if you understand why it might sometimes\n"
 " be necessary:\n"
 "  --trim-centrics            Remove reflections which are duplicated in the\n"
@@ -283,6 +288,7 @@ static RefList *expand_reflections(RefList *in, const SymOpList *initial,
 	RefListIterator *iter;
 	RefList *out;
 	SymOpMask *m;
+	int phase_warning = 0;
 
 	if ( !is_subgroup(initial, target) ) {
 		ERROR("%s is not a subgroup of %s!\n", symmetry_name(target),
@@ -325,10 +331,17 @@ static RefList *expand_reflections(RefList *in, const SymOpList *initial,
 			copy = add_refl(out, he, ke, le);
 			copy_data(copy, refl);
 
-			/* FIXME: Make phase negative if the reflection is
-			 * separated from the original via an inversion */
 			ph = get_phase(refl, &have_phase);
-			if ( have_phase ) set_phase(copy, -ph);
+			if ( have_phase ) {
+				set_phase(copy, ph);
+				if ( !phase_warning ) {
+					ERROR("WARNING: get_hkl can't expand "
+					      "phase values correctly when the "
+					      "structure contains glides or "
+					      "screw axes.\n");
+					phase_warning = 1;
+				}
+			}
 
 		}
 
@@ -409,10 +422,13 @@ int main(int argc, char *argv[])
 	double cutiso = 0.0;
 	float cutn1, cutn2, cutn3;
 	char *pdb = NULL;
+	char *reindex_str = NULL;
+	SymOpList *reindex = NULL;
 
 	/* Long options */
 	const struct option longopts[] = {
 		{"help",               0, NULL,               'h'},
+		{"version",            0, NULL,                5 },
 		{"template",           1, NULL,               't'},
 		{"poisson",            0, &config_poisson,     1},
 		{"noise",              0, &config_noise,       1},
@@ -427,6 +443,7 @@ int main(int argc, char *argv[])
 		{"no-need-all-parts",  0, &config_nap,         0},
 		{"adu-per-photon",     1, NULL,                2},
 		{"cutoff-angstroms",   1, NULL,                3},
+		{"reindex",            1, NULL,                4},
 		{0, 0, NULL, 0}
 	};
 
@@ -440,6 +457,11 @@ int main(int argc, char *argv[])
 			show_help(argv[0]);
 			return 0;
 
+			case 5 :
+			printf("CrystFEL: " CRYSTFEL_VERSIONSTRING "\n");
+			printf(CRYSTFEL_BOILERPLATE"\n");
+			return 0;
+
 			case 't' :
 			template = strdup(optarg);
 			break;
@@ -477,6 +499,10 @@ int main(int argc, char *argv[])
 			cutoff_str = strdup(optarg);
 			break;
 
+			case 4 :
+			reindex_str = strdup(optarg);
+			break;
+
 			case 0 :
 			break;
 
@@ -555,6 +581,12 @@ int main(int argc, char *argv[])
 		expand = NULL;
 	}
 
+	if ( reindex_str != NULL ) {
+		reindex = parse_symmetry_operations(reindex_str);
+		if ( reindex == NULL ) return 1;
+		set_symmetry_name(reindex, "Reindex");
+	}
+
 	if ( (expand != NULL) || (holo != NULL) || config_trimc
 	  || config_multi ) {
 		if ( mero == NULL ) {
@@ -778,6 +810,32 @@ int main(int argc, char *argv[])
 
 	}
 
+	if ( reindex != NULL ) {
+
+		RefList *n;
+		Reflection *refl;
+		RefListIterator *iter;
+
+		n = reflist_new();
+
+		for ( refl = first_refl(input, &iter);
+		      refl != NULL;
+		      refl = next_refl(refl, iter) )
+		{
+			signed int h, k, l;
+			Reflection *rn;
+
+			get_indices(refl, &h, &k, &l);
+			get_equiv(reindex, NULL, 0, h, k, l, &h, &k, &l);
+			rn = add_refl(n, h, k, l);
+			copy_data(rn, refl);
+
+		}
+
+		reflist_free(input);
+		input = n;
+
+	}
 
 	write_reflist(output, input);
 
