@@ -3,13 +3,13 @@
  *
  * Sandbox for indexing
  *
- * Copyright © 2012-2013 Deutsches Elektronen-Synchrotron DESY,
+ * Copyright © 2012-2014 Deutsches Elektronen-Synchrotron DESY,
  *                       a research centre of the Helmholtz Association.
  * Copyright © 2012 Richard Kirian
  * Copyright © 2012 Lorenzo Galli
  *
  * Authors:
- *   2010-2013 Thomas White <taw@physics.org>
+ *   2010-2014 Thomas White <taw@physics.org>
  *   2011      Richard Kirian
  *   2012      Lorenzo Galli
  *   2012      Chunhong Yoon
@@ -74,8 +74,8 @@ struct sb_reader
 	FILE **fhs;
 	int *fds;
 
-	/* Final output fd */
-	int ofd;
+	/* Final output */
+	Stream *stream;
 };
 
 
@@ -90,6 +90,7 @@ struct sandbox
 	int n_hadcrystals_last_stats;
 	int n_crystals_last_stats;
 	int t_last_stats;
+	int suspend_stats;
 
 	struct index_args *iargs;
 
@@ -124,8 +125,7 @@ static void unlock_sandbox(struct sandbox *sb)
 }
 
 
-static char *get_pattern(FILE *fh, char **use_this_one_instead,
-                         int config_basename, const char *prefix)
+static char *get_pattern(FILE *fh, int config_basename, const char *prefix)
 {
 	char *line;
 	char *filename;
@@ -134,22 +134,13 @@ static char *get_pattern(FILE *fh, char **use_this_one_instead,
 	do {
 
 		/* Get the next filename */
-		if ( *use_this_one_instead != NULL ) {
-
-			line = *use_this_one_instead;
-			*use_this_one_instead = NULL;
-
-		} else {
-
-			char *rval;
-
-			line = malloc(1024*sizeof(char));
-			rval = fgets(line, 1023, fh);
-			if ( rval == NULL ) {
-				free(line);
-				return NULL;
-			}
+		char *rval;
 
+		line = malloc(1024*sizeof(char));
+		rval = fgets(line, 1023, fh);
+		if ( rval == NULL ) {
+			free(line);
+			return NULL;
 		}
 
 		chomp(line);
@@ -228,7 +219,8 @@ static void run_work(const struct index_args *iargs,
 			pargs.filename = line;
 			pargs.n_crystals = 0;
 
-			process_image(iargs, &pargs, st, cookie, tmpdir);
+			process_image(iargs, &pargs, st, cookie, tmpdir,
+			              results_pipe);
 
 			/* Request another image */
 			c = sprintf(buf, "%i\n", pargs.n_crystals);
@@ -386,6 +378,7 @@ static void remove_pipe(struct sb_reader *rd, int d)
 static void *run_reader(void *rdv)
 {
 	struct sb_reader *rd = rdv;
+	const int ofd = get_stream_fd(rd->stream);
 
 	while ( 1 ) {
 
@@ -435,7 +428,7 @@ static void *run_reader(void *rdv)
 
 			/* If the chunk cannot be read, assume the connection
 			 * is broken and that the process will die soon. */
-			if ( pump_chunk(rd->fhs[i], rd->ofd) ) {
+			if ( pump_chunk(rd->fhs[i], ofd) ) {
 				/* remove_pipe() assumes that the caller is
 				 * holding rd->lock ! */
 				remove_pipe(rd, i);
@@ -450,8 +443,7 @@ static void *run_reader(void *rdv)
 }
 
 
-static void start_worker_process(struct sandbox *sb, int slot,
-                                 int argc, char *argv[])
+static void start_worker_process(struct sandbox *sb, int slot)
 {
 	pid_t p;
 	int filename_pipe[2];
@@ -547,8 +539,6 @@ static void start_worker_process(struct sandbox *sb, int slot,
 		close(result_pipe[0]);
 
 		st = open_stream_fd_for_write(stream_pipe[1]);
-		write_command(st, argc, argv);
-		write_line(st, "FLUSH");
 		run_work(sb->iargs, filename_pipe[0], result_pipe[1],
 		         st, slot, tmp);
 		close_stream(st);
@@ -616,7 +606,7 @@ static void handle_zombie(struct sandbox *sb)
 				STATUS("Last filename was: %s\n",
 				       sb->last_filename[i]);
 				sb->n_processed++;
-				start_worker_process(sb, i, 0, NULL);
+				start_worker_process(sb, i);
 			}
 
 		}
@@ -627,8 +617,8 @@ static void handle_zombie(struct sandbox *sb)
 
 
 void create_sandbox(struct index_args *iargs, int n_proc, char *prefix,
-                    int config_basename, FILE *fh, char *use_this_one_instead,
-                    int ofd, int argc, char *argv[], const char *tempdir)
+                    int config_basename, FILE *fh,
+                    Stream *stream, const char *tempdir)
 {
 	int i;
 	int allDone;
@@ -662,12 +652,13 @@ void create_sandbox(struct index_args *iargs, int n_proc, char *prefix,
 	sb->n_hadcrystals_last_stats = 0;
 	sb->n_crystals_last_stats = 0;
 	sb->t_last_stats = get_monotonic_seconds();
+	sb->suspend_stats = 0;
 	sb->n_proc = n_proc;
 	sb->iargs = iargs;
 
 	sb->reader->fds = NULL;
 	sb->reader->fhs = NULL;
-	sb->reader->ofd = ofd;
+	sb->reader->stream = stream;
 
 	sb->stream_pipe_write = calloc(n_proc, sizeof(int));
 	if ( sb->stream_pipe_write == NULL ) {
@@ -752,7 +743,7 @@ void create_sandbox(struct index_args *iargs, int n_proc, char *prefix,
 	/* Fork the right number of times */
 	lock_sandbox(sb);
 	for ( i=0; i<n_proc; i++ ) {
-		start_worker_process(sb, i, argc, argv);
+		start_worker_process(sb, i);
 	}
 	unlock_sandbox(sb);
 
@@ -836,25 +827,37 @@ void create_sandbox(struct index_args *iargs, int n_proc, char *prefix,
 
 			chomp(results);
 
-			strtol(results, &eptr, 10);
-			if ( eptr == results ) {
-				if ( strlen(results) > 0 ) {
-					ERROR("Invalid result '%s'\n", results);
+			if ( strcmp(results, "SUSPEND") == 0 ) {
+				sb->suspend_stats++;
+			} else if ( strcmp(results, "RELEASE") == 0 ) {
+				if ( sb->suspend_stats > 0 ) {
+					sb->suspend_stats--;
+				} else {
+					ERROR("RELEASE before SUSPEND.\n");
 				}
 			} else {
 
-				int nc = atoi(results);
-				sb->n_crystals += nc;
-				if ( nc > 0 ) {
-					sb->n_hadcrystals++;
+				strtol(results, &eptr, 10);
+				if ( eptr == results ) {
+					if ( strlen(results) > 0 ) {
+						ERROR("Invalid result '%s'\n",
+						      results);
+					}
+				} else {
+
+					int nc = atoi(results);
+					sb->n_crystals += nc;
+					if ( nc > 0 ) {
+						sb->n_hadcrystals++;
+					}
+					sb->n_processed++;
+
 				}
-				sb->n_processed++;
 
 			}
 
 			/* Send next filename */
-			nextImage = get_pattern(fh, &use_this_one_instead,
-	                                        config_basename, prefix);
+			nextImage = get_pattern(fh, config_basename, prefix);
 
 			free(sb->last_filename[i]);
 			sb->last_filename[i] = nextImage;
@@ -880,7 +883,9 @@ void create_sandbox(struct index_args *iargs, int n_proc, char *prefix,
 		/* Update progress */
 		lock_sandbox(sb);
 		tNow = get_monotonic_seconds();
-		if ( tNow >= sb->t_last_stats+STATS_EVERY_N_SECONDS ) {
+		if ( !sb->suspend_stats
+		    && (tNow >= sb->t_last_stats+STATS_EVERY_N_SECONDS) )
+		{
 
 			STATUS("%4i indexable out of %4i processed (%4.1f%%), "
 			       "%4i crystals so far. "
@@ -935,8 +940,9 @@ void create_sandbox(struct index_args *iargs, int n_proc, char *prefix,
 	pthread_mutex_destroy(&sb->lock);
 
 	STATUS("Final:"
-	       " %i images processed, %i had crystals, %i crystals overall.\n",
-	       sb->n_processed, sb->n_hadcrystals, sb->n_crystals);
+	       " %i images processed, %i had crystals (%.1f%%), %i crystals overall.\n",
+	       sb->n_processed, sb->n_hadcrystals,
+	       100.0 * sb->n_hadcrystals / sb->n_processed, sb->n_crystals);
 
 	free(sb);
 }
