@@ -54,6 +54,7 @@
 #include <sys/time.h>
 #endif
 
+#include "version.h"
 #include "utils.h"
 #include "hdf5-file.h"
 #include "index.h"
@@ -78,6 +79,7 @@ static void show_help(const char *s)
 "Process and index FEL diffraction images.\n"
 "\n"
 " -h, --help               Display this help message.\n"
+"     --version            Print CrystFEL version number and exit.\n"
 "\n"
 " -i, --input=<filename>   Specify file containing list of images to process.\n"
 "                           '-' means stdin, which is the default.\n"
@@ -122,11 +124,13 @@ static void show_help(const char *s)
 "    --min-snr=<n>       Minimum signal-to-noise ratio for peaks.\n"
 "                         Default: 5.\n"
 "    --check-hdf5-snr    Check SNR for peaks from --peaks=hdf5.\n"
+"    --peak-radius=<r>   Integration radii for peak search.\n"
 "    --int-radius=<r>    Set the integration radii.  Default: 4,5,7.\n"
 "-e, --image=<element>   Use this image from the HDF5 file.\n"
 "                          Example: /data/data0.\n"
 "                          Default: The first one found.\n"
 "    --push-res=<n>      Integrate higher than apparent resolution cutoff.\n"
+"    --highres=<n>       Absolute resolution cutoff in Angstroms.\n"
 "\n"
 "\nFor time-resolved stuff, you might want to use:\n\n"
 "     --copy-hdf5-field <f>  Copy the value of field <f> into the stream. You\n"
@@ -176,8 +180,7 @@ int main(int argc, char *argv[])
 	char *filename = NULL;
 	char *outfile = NULL;
 	FILE *fh;
-	int ofd;
-	char *rval = NULL;
+	Stream *st;
 	int config_checkprefix = 1;
 	int config_basename = 0;
 	int integrate_saturated = 0;
@@ -189,14 +192,13 @@ int main(int argc, char *argv[])
 	char *speaks = NULL;
 	char *toler = NULL;
 	int n_proc = 1;
-	char *prepare_line;
-	char prepare_filename[1024];
-	char *use_this_one_instead;
 	struct index_args iargs;
 	char *intrad = NULL;
+	char *pkrad = NULL;
 	char *int_str = NULL;
 	char *tempdir = NULL;
 	char *int_diag = NULL;
+	char *geom_filename = NULL;
 
 	/* Defaults */
 	iargs.cell = NULL;
@@ -217,6 +219,9 @@ int main(int argc, char *argv[])
 	iargs.element = NULL;
 	iargs.hdf5_peak_path = strdup("/processing/hitfinder/peakinfo");
 	iargs.copyme = NULL;
+	iargs.pk_inn = -1.0;
+	iargs.pk_mid = -1.0;
+	iargs.pk_out = -1.0;
 	iargs.ir_inn = 4.0;
 	iargs.ir_mid = 5.0;
 	iargs.ir_out = 7.0;
@@ -234,6 +239,7 @@ int main(int argc, char *argv[])
 	iargs.ipriv = NULL;  /* No default */
 	iargs.int_meth = integration_method("rings-nocen", NULL);
 	iargs.push_res = 0.0;
+	iargs.highres = +INFINITY;
 
 	/* Long options */
 	const struct option longopts[] = {
@@ -251,6 +257,7 @@ int main(int argc, char *argv[])
 		{"image",              1, NULL,               'e'},
 
 		/* Long-only options with no arguments */
+		{"version",            0, NULL,                     99},
 		{"filter-noise",       0, &iargs.noisefilter,        1},
 		{"no-check-prefix",    0, &config_checkprefix,       0},
 		{"basename",           0, &config_basename,          1},
@@ -286,6 +293,8 @@ int main(int argc, char *argv[])
 		{"int-diag",           1, NULL,               18},
 		{"push-res",           1, NULL,               19},
 		{"res-push",           1, NULL,               19}, /* compat */
+		{"peak-radius",        1, NULL,               20},
+		{"highres",            1, NULL,               21},
 
 		{0, 0, NULL, 0}
 	};
@@ -300,6 +309,11 @@ int main(int argc, char *argv[])
 			show_help(argv[0]);
 			return 0;
 
+			case 99 :
+			printf("CrystFEL: " CRYSTFEL_VERSIONSTRING "\n");
+			printf(CRYSTFEL_BOILERPLATE"\n");
+			return 0;
+
 			case 'i' :
 			filename = strdup(optarg);
 			break;
@@ -325,6 +339,7 @@ int main(int argc, char *argv[])
 			break;
 
 			case 'g' :
+			geom_filename = optarg;
 			iargs.det = get_detector_geometry(optarg);
 			if ( iargs.det == NULL ) {
 				ERROR("Failed to read detector geometry from "
@@ -426,6 +441,19 @@ int main(int argc, char *argv[])
 			iargs.push_res *= 1e9;  /* nm^-1 -> m^-1 */
 			break;
 
+			case 20 :
+			pkrad = strdup(optarg);
+			break;
+
+			case 21 :
+			if ( sscanf(optarg, "%f", &iargs.highres) != 1 ) {
+				ERROR("Invalid value for --highres\n");
+				return 1;
+			}
+			/* A -> m^-1 */
+			iargs.highres = 1.0 / (iargs.highres/1e10);
+			break;
+
 			case 0 :
 			break;
 
@@ -547,6 +575,23 @@ int main(int argc, char *argv[])
 		       " probably not appropriate for your patterns.\n");
 	}
 
+	if ( pkrad != NULL ) {
+		int r;
+		r = sscanf(pkrad, "%f,%f,%f",
+		           &iargs.pk_inn, &iargs.pk_mid, &iargs.pk_out);
+		if ( r != 3 ) {
+			ERROR("Invalid parameters for '--peak-radius'\n");
+			return 1;
+		}
+		free(pkrad);
+	}
+
+	if ( iargs.pk_inn < 0.0 ) {
+		iargs.pk_inn = iargs.ir_inn;
+		iargs.pk_mid = iargs.ir_mid;
+		iargs.pk_out = iargs.ir_out;
+	}
+
 	if ( iargs.det == NULL ) {
 		ERROR("You need to provide a geometry file (please read the"
 		      " manual for more details).\n");
@@ -623,35 +668,17 @@ int main(int argc, char *argv[])
 
 	}
 
-	ofd = open(outfile, O_CREAT | O_TRUNC | O_WRONLY, S_IRUSR | S_IWUSR);
-	if ( ofd == -1 ) {
+	st = open_stream_for_write_2(outfile, geom_filename, argc, argv);
+	if ( st == NULL ) {
 		ERROR("Failed to open stream '%s'\n", outfile);
 		return 1;
 	}
 	free(outfile);
 
-	/* Get first filename and use it to set up the indexing */
-	prepare_line = malloc(1024);
-	rval = fgets(prepare_line, 1023, fh);
-	if ( rval == NULL ) {
-		ERROR("Failed to get filename to prepare indexing.\n");
-		return 1;
-	}
-	use_this_one_instead = strdup(prepare_line);
-	chomp(prepare_line);
-	if ( config_basename ) {
-		char *tmp;
-		tmp = safe_basename(prepare_line);
-		free(prepare_line);
-		prepare_line = tmp;
-	}
-	snprintf(prepare_filename, 1023, "%s%s", prefix, prepare_line);
-	free(prepare_line);
-
 	/* Prepare the indexer */
 	if ( indm != NULL ) {
-		ipriv = prepare_indexing(indm, iargs.cell, prepare_filename,
-		                         iargs.det, iargs.beam, iargs.tols);
+		ipriv = prepare_indexing(indm, iargs.cell, iargs.det,
+		                         iargs.beam, iargs.tols);
 		if ( ipriv == NULL ) {
 			ERROR("Failed to prepare indexing.\n");
 			return 1;
@@ -666,7 +693,7 @@ int main(int argc, char *argv[])
 	iargs.ipriv = ipriv;
 
 	create_sandbox(&iargs, n_proc, prefix, config_basename, fh,
-	               use_this_one_instead, ofd, argc, argv, tempdir);
+	               st, tempdir);
 
 	free(prefix);
 	free(tempdir);
