@@ -41,6 +41,7 @@
 #include <pthread.h>
 #include <gsl/gsl_rng.h>
 
+#include "version.h"
 #include "image.h"
 #include "utils.h"
 #include "reflist-utils.h"
@@ -170,11 +171,10 @@ static void calculate_partials(Crystal *cr,
 
 
 static void draw_and_write_image(struct image *image, RefList *reflections,
-                                 gsl_rng *rng)
+                                 gsl_rng *rng, double background)
 {
 	Reflection *refl;
 	RefListIterator *iter;
-	const int background = 3000;
 	int i;
 
 	image->data = calloc(image->width*image->height, sizeof(float));
@@ -215,7 +215,8 @@ static void show_help(const char *s)
 	printf(
 "Generate a stream containing partials from a reflection list.\n"
 "\n"
-" -h, --help              Display this help message.\n"
+" -h, --help               Display this help message.\n"
+"     --version            Print CrystFEL version number and exit.\n"
 "\n"
 "You need to provide the following basic options:\n"
 " -i, --input=<file>       Read reflections from <file>.\n"
@@ -234,7 +235,8 @@ static void show_help(const char *s)
 "     --osf-stddev=<val>   Standard deviation of the scaling factors.\n"
 "     --full-stddev=<val>  Standard deviation of the randomly\n"
 "                           generated full intensities, if not using -i.\n"
-"     --noise-stddev=<val>  Set the standard deviation of the noise.\n"
+"     --noise-stddev=<val> Set the standard deviation of the noise.\n"
+"     --background=<val>   Background level in photons.  Default 3000.\n"
 "\n"
 );
 }
@@ -256,6 +258,7 @@ struct queue_args
 	double osf_stddev;
 	double full_stddev;
 	double noise_stddev;
+	double background;
 
 	struct image *template_image;
 	double max_q;
@@ -349,7 +352,7 @@ static void run_job(void *vwargs, int cookie)
 		snprintf(wargs->image.filename, 255, "dummy.h5");
 	}
 
-	reflections = find_intersections(&wargs->image, cr);
+	reflections = find_intersections(&wargs->image, cr, PMODEL_SPHERE);
 	crystal_set_reflections(cr, reflections);
 
 	for ( i=0; i<NBINS; i++ ) {
@@ -367,7 +370,7 @@ static void run_job(void *vwargs, int cookie)
 
 	if ( qargs->image_prefix != NULL ) {
 		draw_and_write_image(&wargs->image, reflections,
-		                     qargs->rngs[cookie]);
+		                     qargs->rngs[cookie], qargs->background);
 	}
 
 	/* Give a slightly incorrect cell in the stream */
@@ -431,6 +434,7 @@ int main(int argc, char *argv[])
 	double osf_stddev = 2.0;
 	double full_stddev = 1000.0;
 	double noise_stddev = 20.0;
+	double background = 3000.0;
 	gsl_rng *rng_for_seeds;
 	int config_random = 0;
 	char *image_prefix = NULL;
@@ -438,6 +442,7 @@ int main(int argc, char *argv[])
 	/* Long options */
 	const struct option longopts[] = {
 		{"help",               0, NULL,               'h'},
+		{"version",            0, NULL,                8 },
 		{"output",             1, NULL,               'o'},
 		{"input",              1, NULL,               'i'},
 		{"beam",               1, NULL,               'b'},
@@ -452,6 +457,7 @@ int main(int argc, char *argv[])
 		{"full-stddev",        1, NULL,                4},
 		{"noise-stddev",       1, NULL,                5},
 		{"images",             1, NULL,                6},
+		{"background",         1, NULL,                7},
 
 		{"really-random",      0, &config_random,      1},
 
@@ -468,6 +474,11 @@ int main(int argc, char *argv[])
 			show_help(argv[0]);
 			return 0;
 
+			case 8 :
+			printf("CrystFEL: " CRYSTFEL_VERSIONSTRING "\n");
+			printf(CRYSTFEL_BOILERPLATE"\n");
+			return 0;
+
 			case 'i' :
 			input_file = strdup(optarg);
 			break;
@@ -559,6 +570,19 @@ int main(int argc, char *argv[])
 			image_prefix = strdup(optarg);
 			break;
 
+			case 7 :
+			background = strtod(optarg, &rval);
+			if ( *rval != '\0' ) {
+				ERROR("Invalid background level.\n");
+				return 1;
+			}
+			if ( background < 0.0 ) {
+				ERROR("Invalid background level.");
+				ERROR(" (must be positive).\n");
+				return 1;
+			}
+			break;
+
 			case 0 :
 			break;
 
@@ -622,7 +646,6 @@ int main(int argc, char *argv[])
 		ERROR("Failed to read geometry from '%s'\n", geomfile);
 		return 1;
 	}
-	free(geomfile);
 
 	if ( sym_str == NULL ) sym_str = strdup("1");
 	sym = get_pointgroup(sym_str);
@@ -633,6 +656,8 @@ int main(int argc, char *argv[])
 	/* Load (full) reflections */
 	if ( input_file != NULL ) {
 
+		RefList *as;
+
 		full = read_reflections(input_file);
 		if ( full == NULL ) {
 			ERROR("Failed to read reflections from '%s'\n",
@@ -646,6 +671,10 @@ int main(int argc, char *argv[])
 			return 1;
 		}
 
+		as = asymmetric_indices(full, sym);
+		reflist_free(full);
+		full = as;
+
 	} else {
 		random_intensities = 1;
 	}
@@ -659,7 +688,7 @@ int main(int argc, char *argv[])
 		ERROR("You must give a filename for the output.\n");
 		return 1;
 	}
-	stream = open_stream_for_write(output_file);
+	stream = open_stream_for_write_2(output_file, geomfile, argc, argv);
 	if ( stream == NULL ) {
 		ERROR("Couldn't open output file '%s'\n", output_file);
 		return 1;
@@ -701,6 +730,7 @@ int main(int argc, char *argv[])
 	qargs.osf_stddev = osf_stddev;
 	qargs.full_stddev = full_stddev;
 	qargs.noise_stddev = noise_stddev;
+	qargs.background = background;
 	qargs.max_q = largest_q(&image);
 	qargs.image_prefix = image_prefix;
 
@@ -814,6 +844,7 @@ int main(int argc, char *argv[])
 	free_symoplist(sym);
 	reflist_free(full);
 	free(save_file);
+	free(geomfile);
 
 	return 0;
 }
