@@ -40,6 +40,7 @@
 #include <unistd.h>
 #include <getopt.h>
 
+#include "version.h"
 #include "image.h"
 #include "diffraction.h"
 #include "diffraction-gpu.h"
@@ -65,6 +66,7 @@ static void show_help(const char *s)
 "pulses of X-rays from a free electron laser.\n"
 "\n"
 " -h, --help                Display this help message.\n"
+"     --version             Print CrystFEL version number and exit.\n"
 "\n"
 " -p, --pdb=<file>          PDB file from which to get the unit cell.\n"
 "                            (The actual Bragg intensities come from the\n"
@@ -97,22 +99,27 @@ static void show_help(const char *s)
 }
 
 
-static double *intensities_from_list(RefList *list)
+static double *intensities_from_list(RefList *list, SymOpList *sym)
 {
 	Reflection *refl;
 	RefListIterator *iter;
 	double *out = new_arr_intensity();
+	SymOpMask *m = new_symopmask(sym);
+	int neq = num_equivs(sym, NULL);
 
 	for ( refl = first_refl(list, &iter);
 	      refl != NULL;
 	      refl = next_refl(refl, iter) ) {
 
 		signed int h, k, l;
+		int eps;
 		double intensity = get_intensity(refl);
 
 		get_indices(refl, &h, &k, &l);
+		special_position(sym, m, h, k, l);
+		eps = neq / num_equivs(sym, m);
 
-		set_arr_intensity(out, h, k, l, intensity);
+		set_arr_intensity(out, h, k, l, intensity / eps);
 
 	}
 
@@ -261,6 +268,7 @@ int main(int argc, char *argv[])
 	/* Long options */
 	const struct option longopts[] = {
 		{"help",               0, NULL,               'h'},
+		{"version",            0, NULL,                7 },
 		{"gpu",                0, &config_gpu,         1},
 		{"random-orientation", 0, NULL,               'r'},
 		{"number",             1, NULL,               'n'},
@@ -297,6 +305,11 @@ int main(int argc, char *argv[])
 			show_help(argv[0]);
 			return 0;
 
+			case 7 :
+			printf("CrystFEL: " CRYSTFEL_VERSIONSTRING "\n");
+			printf(CRYSTFEL_BOILERPLATE"\n");
+			return 0;
+
 			case 'r' :
 			config_randomquat = 1;
 			break;
@@ -513,7 +526,7 @@ int main(int argc, char *argv[])
 		} else {
 			phases = NULL;
 		}
-		intensities = intensities_from_list(reflections);
+		intensities = intensities_from_list(reflections, sym);
 		phases = phases_from_list(reflections);
 		flags = flags_from_list(reflections);
 
