@@ -3,11 +3,11 @@
  *
  * Post refinement
  *
- * Copyright © 2012-2013 Deutsches Elektronen-Synchrotron DESY,
+ * Copyright © 2012-2014 Deutsches Elektronen-Synchrotron DESY,
  *                       a research centre of the Helmholtz Association.
  *
  * Authors:
- *   2010-2013 Thomas White <taw@physics.org>
+ *   2010-2014 Thomas White <taw@physics.org>
  *
  * This file is part of CrystFEL.
  *
@@ -48,43 +48,77 @@
 #include "cell-utils.h"
 
 
+/* Minimum partiality of a reflection for it to be used for refinement */
+#define MIN_PART_REFINE (0.1)
+
 /* Maximum number of iterations of NLSq to do for each image per macrocycle. */
 #define MAX_CYCLES (10)
 
 
-/* Returns dp/dr at "r" */
-static double partiality_gradient(double r, double profile_radius)
+static double dpdq(double r, double profile_radius)
 {
-	double q, dpdq, dqdr;
+	double q;
 
 	/* Calculate degree of penetration */
 	q = (r + profile_radius)/(2.0*profile_radius);
 
-	/* dp/dq */
-	dpdq = 6.0*(q-pow(q, 2.0));
+	return 6.0*(q-q*q);
+}
 
-	/* dq/dr */
-	dqdr = 1.0 / (2.0*profile_radius);
 
-	return dpdq * dqdr;
+/* Returns dp/dr at "r" */
+static double partiality_gradient(double r, double profile_radius,
+                                  PartialityModel pmodel)
+{
+	double dqdr;  /* dq/dr */
+
+	switch ( pmodel ) {
+
+		default:
+		case PMODEL_UNITY:
+		return 0.0;
+
+		case PMODEL_SPHERE:
+		dqdr = 1.0 / (2.0*profile_radius);
+		return dpdq(r, profile_radius) * dqdr;
+
+		case PMODEL_GAUSSIAN:
+		/* FIXME: Get a proper gradient */
+		dqdr = 1.0 / (2.0*profile_radius);
+		return dpdq(r, profile_radius) * dqdr;
+
+		case PMODEL_THIN:
+		return -(2.0*r)/(profile_radius*profile_radius);
+
+	}
 }
 
 
 /* Returns dp/drad at "r" */
-static double partiality_rgradient(double r, double profile_radius)
+static double partiality_rgradient(double r, double profile_radius,
+                                   PartialityModel pmodel)
 {
-	double q, dpdq, dqdrad;
+	double dqdrad;  /* dq/drad */
 
-	/* Calculate degree of penetration */
-	q = (r + profile_radius)/(2.0*profile_radius);
+	switch ( pmodel ) {
 
-	/* dp/dq */
-	dpdq = 6.0*(q-pow(q, 2.0));
+		default:
+		case PMODEL_UNITY:
+		return 0.0;
 
-	/* dq/drad */
-	dqdrad = -0.5 * r * pow(profile_radius, -2.0);
+		case PMODEL_SPHERE:
+		dqdrad = -0.5 * r / (profile_radius * profile_radius);
+		return dpdq(r, profile_radius) * dqdrad;
 
-	return dpdq * dqdrad;
+		case PMODEL_GAUSSIAN:
+		/* FIXME: Get a proper gradient */
+		dqdrad = -0.5 * r / (profile_radius * profile_radius);
+		return dpdq(r, profile_radius) * dqdrad;
+
+		case PMODEL_THIN:
+		return 2.0*r*r*pow(profile_radius, -3.0);
+
+	}
 }
 
 
@@ -144,29 +178,58 @@ double p_gradient(Crystal *cr, int k, Reflection *refl, PartialityModel pmodel)
 
 	/* Calculate the gradient of partiality wrt excitation error. */
 	if ( clamp_low == 0 ) {
-		glow = partiality_gradient(rlow, r);
+		glow = partiality_gradient(rlow, r, pmodel);
 	} else {
 		glow = 0.0;
 	}
 	if ( clamp_high == 0 ) {
-		ghigh = partiality_gradient(rhigh, r);
+		ghigh = partiality_gradient(rhigh, r, pmodel);
 	} else {
 		ghigh = 0.0;
 	}
 
+	if ( k == REF_R ) {
+		switch ( pmodel ) {
+
+			default:
+			case PMODEL_UNITY:
+			return 0.0;
+
+			case PMODEL_GAUSSIAN:
+			gr  = partiality_rgradient(rlow, r, pmodel);
+			gr -= partiality_rgradient(rhigh, r, pmodel);
+			return gr;
+
+			case PMODEL_SPHERE:
+			gr  = partiality_rgradient(rlow, r, pmodel);
+			gr -= partiality_rgradient(rhigh, r, pmodel);
+			return gr;
+
+			case PMODEL_THIN:
+			return 2.0*rlow*rlow/(r*r*r);
+		}
+	}
+
+	if ( k == REF_DIV ) {
+		switch ( pmodel ) {
+
+			default:
+			case PMODEL_UNITY:
+			return 0.0;
+
+			case PMODEL_GAUSSIAN:
+			case PMODEL_SPHERE:
+			return (ds*glow + ds*ghigh) / 2.0;
+
+			case PMODEL_THIN:
+			return 0.0;
+		}
+	}
+
 	/* For many gradients, just multiply the above number by the gradient
 	 * of excitation error wrt whatever. */
 	switch ( k ) {
 
-		case REF_DIV :
-		/* Small angle approximation */
-		return (ds*glow + ds*ghigh) / 2.0;
-
-		case REF_R :
-		gr  = partiality_rgradient(rlow, r);
-		gr -= partiality_rgradient(rhigh, r);
-		return gr;
-
 		/* Cell parameters and orientation */
 		case REF_ASX :
 		return hs * sin(phi) * cos(azi) * (ghigh-glow);
@@ -208,37 +271,16 @@ double l_gradient(Crystal *cr, int k, Reflection *refl, PartialityModel pmodel)
 {
 	double ds;
 	signed int hs, ks, ls;
+	double L;
 
-	switch ( k ) {
-
-		/* Cell parameters do not affect Lorentz factor */
-		case REF_ASX :
-		case REF_BSX :
-		case REF_CSX :
-		case REF_ASY :
-		case REF_BSY :
-		case REF_CSY :
-		case REF_ASZ :
-		case REF_BSZ :
-		case REF_CSZ :
-		return 0.0;
-
-		/* Nor does change of radius */
-		case REF_R :
-		return 0.0;
-
-		default:
-		break;
-
-	}
-
-	assert(k == REF_DIV);
+	if ( k != REF_DIV ) return 0.0;
 
 	get_symmetric_indices(refl, &hs, &ks, &ls);
 
 	ds = 2.0 * resolution(crystal_get_cell(cr), hs, ks, ls);
 
-	return -ds*pow(get_lorentz(refl), 2.0) / LORENTZ_SCALE;
+	L = get_lorentz(refl);
+	return -ds*L*L / LORENTZ_SCALE;
 }
 
 
@@ -408,7 +450,7 @@ static gsl_vector *solve_svd(gsl_vector *v, gsl_matrix *M, int *n_filt,
 	s_vec = gsl_matrix_calloc(n, n);
 	err = gsl_linalg_SV_decomp_jacobi(SAS, s_vec, s_val);
 	if ( err ) {
-		ERROR("SVD failed: %s\n", gsl_strerror(err));
+		if ( verbose ) ERROR("SVD failed: %s\n", gsl_strerror(err));
 		gsl_matrix_free(s_vec);
 		gsl_vector_free(s_val);
 		gsl_matrix_free(SAS);
@@ -450,7 +492,7 @@ static gsl_vector *solve_svd(gsl_vector *v, gsl_matrix *M, int *n_filt,
 
 /* Perform one cycle of post refinement on 'image' against 'full' */
 static double pr_iterate(Crystal *cr, const RefList *full,
-                         PartialityModel pmodel, struct prdata *prdata)
+                         PartialityModel pmodel, int *n_filtered)
 {
 	gsl_matrix *M;
 	gsl_vector *v;
@@ -463,6 +505,8 @@ static double pr_iterate(Crystal *cr, const RefList *full,
 	int nref = 0;
 	const int verbose = 0;
 
+	*n_filtered = 0;
+
 	reflections = crystal_get_reflections(cr);
 
 	M = gsl_matrix_calloc(NUM_PARAMS, NUM_PARAMS);
@@ -482,20 +526,19 @@ static double pr_iterate(Crystal *cr, const RefList *full,
 		Reflection *match;
 		double gradients[NUM_PARAMS];
 
-		if ( !get_refinable(refl) ) continue;
-
 		/* Find the full version */
 		get_indices(refl, &ha, &ka, &la);
 		match = find_refl(full, ha, ka, la);
-		if ( match == NULL ) {
-			ERROR("%3i %3i %3i isn't in the reference list, so why "
-			      " is it marked as refinable?\n", ha, ka, la);
-			continue;
-		}
+		if ( match == NULL ) continue;
+
+		if ( (get_intensity(refl) < 3.0*get_esd_intensity(refl))
+		  || (get_partiality(refl) < MIN_PART_REFINE)
+		  || (get_redundancy(match) < 2) ) continue;
+
 		I_full = get_intensity(match);
 
 		/* Actual measurement of this reflection from this pattern? */
-		I_partial = get_intensity(refl) * crystal_get_osf(cr);
+		I_partial = get_intensity(refl) / crystal_get_osf(cr);
 		p = get_partiality(refl);
 		l = get_lorentz(refl);
 
@@ -549,14 +592,14 @@ static double pr_iterate(Crystal *cr, const RefList *full,
 
 	//STATUS("%i reflections went into the equations.\n", nref);
 	if ( nref == 0 ) {
-		crystal_set_user_flag(cr, 1);
+		crystal_set_user_flag(cr, 2);
 		gsl_matrix_free(M);
 		gsl_vector_free(v);
 		return 0.0;
 	}
 
 	max_shift = 0.0;
-	shifts = solve_svd(v, M, &prdata->n_filtered, verbose);
+	shifts = solve_svd(v, M, n_filtered, verbose);
 	if ( shifts != NULL ) {
 
 		for ( param=0; param<NUM_PARAMS; param++ ) {
@@ -567,7 +610,7 @@ static double pr_iterate(Crystal *cr, const RefList *full,
 		}
 
 	} else {
-		crystal_set_user_flag(cr, 2);
+		crystal_set_user_flag(cr, 3);
 	}
 
 	gsl_matrix_free(M);
@@ -595,7 +638,8 @@ static double guide_dev(Crystal *cr, const RefList *full)
 		Reflection *full_version;
 		double I_full, I_partial;
 
-		if ( !get_refinable(refl) ) continue;
+		if ( (get_intensity(refl) < 3.0*get_esd_intensity(refl))
+		  || (get_partiality(refl) < MIN_PART_REFINE) ) continue;
 
 		get_indices(refl, &h, &k, &l);
 		assert((h!=0) || (k!=0) || (l!=0));
@@ -679,6 +723,12 @@ struct prdata pr_refine(Crystal *cr, const RefList *full,
 	struct prdata prdata;
 	double mean_p_change = 0.0;
 
+	prdata.refined = 0;
+	prdata.n_filtered = 0;
+
+	/* Don't refine crystal if scaling was bad */
+	if ( crystal_get_user_flag(cr) != 0 ) return prdata;
+
 	if ( verbose ) {
 		dev = guide_dev(cr, full);
 		STATUS("\n");  /* Deal with progress bar */
@@ -703,8 +753,7 @@ struct prdata pr_refine(Crystal *cr, const RefList *full,
 		cell_get_reciprocal(crystal_get_cell(cr), &asx, &asy, &asz,
 			               &bsx, &bsy, &bsz, &csx, &csy, &csz);
 
-		prdata.n_filtered = 0;
-		pr_iterate(cr, full, pmodel, &prdata);
+		pr_iterate(cr, full, pmodel, &prdata.n_filtered);
 
 		update_partialities_2(cr, pmodel, &n_gained, &n_lost,
 		                      &mean_p_change);
@@ -719,7 +768,10 @@ struct prdata pr_refine(Crystal *cr, const RefList *full,
 
 		if ( 3*n_lost > n_total ) {
 			revert_crystal(cr, backup);
-			crystal_set_user_flag(cr, 3);
+			update_partialities_2(cr, pmodel, &n_gained, &n_lost,
+			                      &mean_p_change);
+			crystal_set_user_flag(cr, 4);
+			break;
 		}
 
 		i++;
@@ -728,5 +780,9 @@ struct prdata pr_refine(Crystal *cr, const RefList *full,
 
 	free_backup_crystal(backup);
 
+	if ( crystal_get_user_flag(cr) == 0 ) {
+		prdata.refined = 1;
+	}
+
 	return prdata;
 }
