@@ -249,8 +249,6 @@ static void partiality_graph(cairo_t *cr, Crystal **crystals, int n,
 			Reflection *f;
 			int bin;
 
-			if ( !get_scalable(refl) ) continue;
-
 			get_indices(refl, &h, &k, &l);
 			f = find_refl(full, h, k, l);
 			if ( f == NULL ) continue;
@@ -364,8 +362,6 @@ static void partiality_histogram(cairo_t *cr, Crystal **crystals, int n,
 			signed int h, k, l;
 			Reflection *f;
 
-			if ( !get_scalable(refl) ) continue;
-
 			get_indices(refl, &h, &k, &l);
 			f = find_refl(full, h, k, l);
 			if ( f == NULL ) continue;
@@ -604,8 +600,6 @@ static void intensity_histogram(cairo_t *cr, Crystal **crystals, int n,
 		{
 			double Iobs, pcalc, Ifull_est;
 
-			if ( !get_scalable(f) ) continue;
-
 			pcalc = get_partiality(f);
 			Iobs = get_intensity(f);
 			Ifull_est = Iobs / (pcalc * osf);
@@ -639,8 +633,6 @@ static void intensity_histogram(cairo_t *cr, Crystal **crystals, int n,
 		{
 			double Iobs, pcalc, Ifull_est;
 
-			if ( !get_scalable(f) ) continue;
-
 			pcalc = get_partiality(f);
 			Iobs = get_intensity(f);
 			Ifull_est = Iobs / (pcalc * osf);
@@ -896,9 +888,6 @@ void sr_iteration(SRContext *sr, int iteration, struct srdata *d)
 		cairo_restore(sr->cr);
 
 	}
-
-	STATUS("%i filtered total, %5.2f filtered per crystal\n",
-	       d->n_filtered, (double)d->n_filtered / d->n);
 }
 
 
